/****************************************************************************/
/*                                                                          */
/* Sample to show how to use the X-CAN-2 on a MAX6pci                       */
/*                                                                          */
/* In demonstrates the following issues:                                    */
/* - Initalization of the board and download of the MDD                     */
/* - Opening the Bus control channel and configuration of the CAN bus       */
/* - Opening of CAN-Bus channels to transmit and receive data               */
/* - Usage of channel callback functions                                    */
/*                                                                          */
/* Program usage:                                                           */
/* 1. Enter Board ID, Slot No. of X-CAN-2 and desired CAN channel (0 or 1)  */
/*    and click <Start>.                                                    */  
/* 2. Select the desired Bitrate (this sample allows only standard values), */
/*    the format of the CAN-identifier (11 or 29 bit) and if the Bus        */
/*    termination should be switched on, then click <Config Bus>.           */
/* 3. Now transmit and/or receive objects can be created (select the        */
/*    desired properties for the objects before clicking <Create>)          */
/* 4. The transmit object can transmit the data in its Data field by        */
/*    clicking <Transmit Data>.                                             */  
/* 5. The receive object can receive data by clicking the Receive button.   */
/*                                                                          */
/* Transmit and Receive channels are opened with a callback function.       */
/* All message transfers from configured message obejects is displayed.     */
/*                                                                          */
/* Using the module with a X-MAX-1 in a realtime program:                   */
/* Only board initialization and windows specific commands have to be       */
/* changed. Opening and using the MDD channels is the same on all platforms.*/
/*                                                                          */
/* Copyright SORCUS Computer GmbH 2002	                                     */ 
/****************************************************************************/

/*
  History:
  10.09.2002: first release
  20.03.2003: callback functions added
*/

#include "stdafx.h"
#include "CANSMPL.h"
#include "CANSMPLDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CCANSMPLDlg *pDialog;

/////////////////////////////////////////////////////////////////////////////
CCANSMPLDlg::CCANSMPLDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCANSMPLDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCANSMPLDlg)
	m_Board = 0;
	m_Channel = 0;
	m_XCanSlot = 0;
	m_VersionInfo = _T("");
	m_IDFormat = FALSE;
	m_BusResistor = FALSE;
	m_TmtID = 0;
	m_TmtExclBuffer = FALSE;
	m_TmtNoInt = FALSE;
	m_TmtRemote = FALSE;
	m_TmtData = _T("");
	m_RcvID = 0;
	m_RcvExclBuffer = FALSE;
	m_RcvActive = FALSE;
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

/////////////////////////////////////////////////////////////////////////////
void CCANSMPLDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCANSMPLDlg)
	DDX_Control(pDX, IDC_EDIT_RCV_DATA, m_Event);
	DDX_Control(pDX, IDC_COMBO_BITRATE, m_BitTiming);
	DDX_Text(pDX, IDC_EDIT_BOARD_ID, m_Board);
	DDX_Text(pDX, IDC_EDIT_CAN_CHANNEL, m_Channel);
	DDV_MinMaxUInt(pDX, m_Channel, 0, 1);
	DDX_Text(pDX, IDC_EDIT_CAN_SLOT, m_XCanSlot);
	DDV_MinMaxUInt(pDX, m_XCanSlot, 1, 6);
	DDX_Text(pDX, IDC_STATIC_VERSION, m_VersionInfo);
	DDX_Check(pDX, IDC_CHECK_29BIT, m_IDFormat);
	DDX_Check(pDX, IDC_CHECK_RON, m_BusResistor);
	DDX_Text(pDX, IDC_EDIT_TMT_ID, m_TmtID);
	DDX_Check(pDX, IDC_CHECK_TMT_EXCL, m_TmtExclBuffer);
	DDX_Check(pDX, IDC_CHECK_TMT_CALLBACK, m_TmtNoInt);
	DDX_Check(pDX, IDC_CHECK_PAS_TMT, m_TmtRemote);
	DDX_Text(pDX, IDC_EDIT_TMT_DATA, m_TmtData);
	DDV_MaxChars(pDX, m_TmtData, 24);
	DDX_Text(pDX, IDC_EDIT_RCV_ID, m_RcvID);
	DDX_Check(pDX, IDC_CHECK_RCV_EXCL, m_RcvExclBuffer);
	DDX_Check(pDX, IDC_CHECK_RCV_ACT, m_RcvActive);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CCANSMPLDlg, CDialog)
	//{{AFX_MSG_MAP(CCANSMPLDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_BUSCFG, OnButtonBuscfg)
	ON_BN_CLICKED(IDC_BUTTON_TMT_CREATE, OnButtonTmtCreate)
	ON_BN_CLICKED(IDC_BUTTON_TMT, OnButtonTmt)
	ON_BN_CLICKED(IDC_BUTTON_RCV_CREATE, OnButtonRcvCreate)
	ON_BN_CLICKED(IDC_BUTTON_RCV, OnButtonRcv)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
   char		 sErrMsg[200], sMessage[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	sprintf (sMessage, "Error %04Xh in %s (%s)", Error, sText, sErrMsg);
	AfxMessageBox(sMessage);
	max_clear_error();
}

/****************************************************************************/
// Callback function for message object channels
// In this sample one callback function is used for all message object channels.
// It just displays the message object data.
// The message-object ID was written into ulPar when the channel was opened.
// Now this value is passed into ID.
MAX_CALLBACK CCANSMPLDlg::msgcallback(MAXCHLHND handle, ULONG ID, ULONG size, void *pData)
{
   CString  sText, sData, sOldText;
   UCHAR    *pucMsgData;
   USHORT   i;
   
   pucMsgData = (UCHAR*)pData;
   sText.Format("\r\n%08X %08X ", handle, ID);
   for (i=0; i<size; i++)
   {
      sData.Format("%02X ", *pucMsgData);
      sText = sText + sData;
      pucMsgData++;
   }

   pDialog->m_Event.GetWindowText(sOldText);
   pDialog->m_Event.SetWindowText(sOldText + sText);
}

/****************************************************************************/
// callback function for bus errors
// It decodes the error event.
MAX_CALLBACK CCANSMPLDlg::buscallback(MAXCHLHND handle, ULONG par, ULONG size, void *pData)
{
   CString  sText, sOldText, sReason;
   ULONG *pulData, ulReason;
   
   pulData = (ULONG*)pData;
   switch ((*pulData) & 0x00000003)
   {
   case XCAN_BUS_ON: sText = "\r\nBus on";
      break;
   case XCAN_BUS_OFF: sText = "\r\nBus off";
      break;
   case XCAN_BUS_HEAVY: sText = "\r\nBus heavy";
      break;
   default: sText = "";
      break;
   }

   if (*pulData & XCAN_LINE_ERROR)  
   {  // only for the fault tolerant interfaces on X-CAN-2/F or /M
      sText = sText + " (Line error)";
   }
   ulReason = *pulData;
   ulReason >>= 16;
   sReason.Format(" (Reason %X)", ulReason);
   sText = sText + sReason;
   pDialog->m_Event.GetWindowText(sOldText);
   pDialog->m_Event.SetWindowText(sOldText + sText);
}

/////////////////////////////////////////////////////////////////////////////
BOOL CCANSMPLDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			
	SetIcon(m_hIcon, FALSE);		

   pDialog = this;

   // library initialization
	max_init_lib(0);
	
	return TRUE;  
}

/////////////////////////////////////////////////////////////////////////////
void CCANSMPLDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // Gertekontext fr Zeichnen

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Symbol in Client-Rechteck zentrieren
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Symbol zeichnen
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

/////////////////////////////////////////////////////////////////////////////
HCURSOR CCANSMPLDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

/////////////////////////////////////////////////////////////////////////////
// Start Button: Reset and initialize board and load MDD (PC based)
void CCANSMPLDlg::OnButtonStart() 
{
   MAX_ERROR Error;
	MAX_VERSION	Version;
   ULONG       ulDate;
	char			szVersion[40], szDate[40], cModuleRev;
	MAX_MODULINFO_TYPE rcModulInfo;

   UpdateData (TRUE);

   // reset X-Bus 
   Error = max_reset_board(m_Board, 30);
	if (Error) 
   {
		ErrorHandler(Error, "max_reset_board");
		return;
	}

	// get a handle for the Windows CPU (MDD is PC based)
	Error = max_connect_cpu (m_Board, 0, 0, NULL, &m_hCPU);
	if (Error) 
   {
		ErrorHandler(Error, "max_connect_cpu");
		return;
	}

   // load MDD for the X-CAN-2
	Error = max_load_mdd (m_hCPU, m_XCanSlot, 0, 
                         0, MDD_X_CAN_2, NULL, 
								 &m_hXCanMdd);
	if (Error) 
   {
		ErrorHandler(Error, "max_load_mdd");
		return;
	}
   
   // display info about module and MDD
	Error = max_get_module_info(m_Board, m_XCanSlot, 0, &rcModulInfo);
   cModuleRev = (char)rcModulInfo.usVersion;
   max_get_mdd_version(m_hXCanMdd, &Version, &ulDate);
   max_get_version_string(Version, ulDate, szVersion, szDate);
   m_VersionInfo.Format("MDD-Version: %s \nModule Rev. %c", szVersion, cModuleRev);
   
   // Init Controls with default settings
   m_BitTiming.SetCurSel(3);  // 125 kBit/s
   m_BusResistor = TRUE;      // Bus resistor is switched on
   m_IDFormat = FALSE;        // use 11 Bit IDs
   m_Event.SetWindowText("Handle   ID       Data");

   UpdateData (FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// Bus Config Button: Open a bus control channel and configure CAN bus
void CCANSMPLDlg::OnButtonBuscfg() 
{
	CPS_XCAN		cps;
	MAX_ERROR	Error;
	ULONG			size = sizeof(XCAN_BIT_PARAMS);
	XCAN_BIT_PARAMS Timing;
   MAXCHLHND   hBusCtrl;

	UpdateData(TRUE); 

	cps.usDevice     = DEVICE_CTRL;
   cps.usIndexFirst = XCAN_BUS_CTRL_INDEX;
   cps.usIndexLast  = cps.usIndexFirst;
	cps.usChannel    = m_Channel;
	cps.usFlags      = 0;

	Error = max_open_channel(m_hXCanMdd, sizeof(cps), &cps, buscallback, 0, &hBusCtrl);
   if (Error)
   {
		ErrorHandler(Error, "max_open_channel (Bus Control)");
      return;
   }

   // make configuration settings now and configure CAN bus
   if (m_BusResistor)
	   Timing.Rbus = 1;
   else
	   Timing.Rbus = 1;
   if (m_IDFormat)
   	Timing.MsgFormat = 29;
   else
	   Timing.MsgFormat = 11;

	switch (m_BitTiming.GetCurSel())
	{
	case 0:	// 10 kBit/s
		Timing.Tq = 5000;
		Timing.Tseg1 = 16;
		Timing.Tseg2 = 3;
		Timing.Tsjw = 2;
		break;
	case 1:	// 20 kBit/s
		Timing.Tq = 2500;
		Timing.Tseg1 = 16;
		Timing.Tseg2 = 3;
		Timing.Tsjw = 2;
		break;
	case 2:	// 50 kBit/s
		Timing.Tq = 1000;
		Timing.Tseg1 = 16;
		Timing.Tseg2 = 3;
		Timing.Tsjw = 2;
		break;
	case 3:	// 125 kBit/s
		Timing.Tq = 500;
		Timing.Tseg1 = 13;
		Timing.Tseg2 = 2;
		Timing.Tsjw = 1;
		break;
	case 4:	// 250 kBit/s
		Timing.Tq = 250;
		Timing.Tseg1 = 13;
		Timing.Tseg2 = 2;
		Timing.Tsjw = 1;
		break;
	case 5:	// 500 kBit/s
		Timing.Tq = 125;
		Timing.Tseg1 = 13;
		Timing.Tseg2 = 2;
		Timing.Tsjw = 1;
		break;
	case 6:	// 800 kBit/s
		Timing.Tq = 125;
		Timing.Tseg1 = 7;
		Timing.Tseg2 = 2;
		Timing.Tsjw = 1;
		break;
	case 7:	// 800 kBit/s
		Timing.Tq = 125;
		Timing.Tseg1 = 5;
		Timing.Tseg2 = 2;
		Timing.Tsjw = 1;
		break;
	default:		AfxMessageBox("Illegal value");
		return;
	}
	Timing.SpB	= 1;  
	Error = max_write_channel_block(hBusCtrl, &size, &Timing);
	if (Error)
		ErrorHandler(Error, "max_write_channel_block (Bus Parameter)");

   // the bus control channel is not needed any longer : close it
	Error = max_close_channel (&hBusCtrl);
	if (Error)
		ErrorHandler(Error, "max_close_channel (Bus)");
}

/////////////////////////////////////////////////////////////////////////////
// Create Transmit Object button: Open a channel to a transmit object
void CCANSMPLDlg::OnButtonTmtCreate() 
{
	CPS_XCAN		cps;
	MAX_ERROR	Error;

	UpdateData(TRUE); 

	cps.usDevice  = DEVICE_BUS_OUT;
	cps.usChannel = m_Channel;
   cps.ulCanID   = m_TmtID;
	cps.usFlags   = 0;
   cps.usTimeout = 10000;
   if (m_TmtRemote)  // this is a passive transmit object
      cps.usFlags |= XCAN_REMOTE;
   if (m_TmtNoInt)
      cps.usFlags |= XCAN_NO_INT;
   if (m_TmtExclBuffer)
      cps.usFlags |= XCAN_EXCL_BUFFER;

   // The CAN-ID is written into the parameter ulPar. 
   // This value will be given to the callback function when it is called.
   // In this way the CAN-ID is available very easily within the callback function.
   // If you want to use ulPar for another purpose and you need the ID
   // within the callback function you can retrieve the ID with:
   // max_channel_info(hChannel, INFO_XCAN_CAN_ID, &s, &ID);

	Error = max_open_channel(m_hXCanMdd, sizeof(cps), &cps, 
                            msgcallback, m_TmtID, &m_hTransmitter);
   if (Error)
		ErrorHandler(Error, "max_open_channel (Transmit Object)");
}

/////////////////////////////////////////////////////////////////////////////
// Transmit Button: Transmit data of transmit object
void CCANSMPLDlg::OnButtonTmt() 
{
	char	      *sData;
   int         i, k;
   UCHAR       aucTmtData[16];
   MAX_ERROR   Error;
   ULONG       size;

   UpdateData(TRUE); 
   
   // write input data into array
   // expected input Format: 12 FF 67 99 34 AB 37 D0 
   // max 8 hex. numbers separated with blanks:
   i = 0;
   k = 0;
   sData = m_TmtData.GetBuffer(24);
   do
   {
	   sscanf(sData + i, "%X", &aucTmtData[k]);
      i++;                          // jump over the blank
      k++;                          // next position in the array
      i = m_TmtData.Find(' ', i);   // next blank
   }
   while ((i != -1) && (k <= 8));

   size = k;
	Error = max_write_channel_block(m_hTransmitter, &size, aucTmtData);
	if (Error)
		ErrorHandler(Error, "max_write_channel_block (Transmit)");
}

/////////////////////////////////////////////////////////////////////////////
// Create Receive Object Button: Open a channel to a receive object
void CCANSMPLDlg::OnButtonRcvCreate() 
{
	CPS_XCAN		cps;
	MAX_ERROR	Error;

	UpdateData(TRUE); 

	cps.usDevice = DEVICE_BUS_IN;
	cps.usChannel = m_Channel;
   cps.ulCanID = m_RcvID;
	cps.usFlags = 0;
   cps.usTimeout = 10000;
   if (m_RcvActive)  // this is a active receive object
      cps.usFlags |= (XCAN_REMOTE + XCAN_NO_INT);
   if (m_RcvExclBuffer)
      cps.usFlags |= XCAN_EXCL_BUFFER;

   // The CAN-ID is written into the parameter ulPar. 
   // This value will be given to the callback function when it is called.
   // In this way the CAN-ID is available very easily within the callback function.
	Error = max_open_channel(m_hXCanMdd, sizeof(cps), &cps, 
                            msgcallback, m_RcvID, &m_hReceiver);
   if (Error)
		ErrorHandler(Error, "max_open_channel (Receive Object)");
}

/////////////////////////////////////////////////////////////////////////////
// Receive Button: Read data of receive object and display it
void CCANSMPLDlg::OnButtonRcv() 
{
	MAX_ERROR	Error;
	UCHAR			data[8] = {0,0,0,0,0,0,0,0}, i;
	ULONG			size = 8;
	CString		Text, Line;

	UpdateData(TRUE); 
	Error = max_read_channel_block(m_hReceiver, &size, data);
	if (Error)
		ErrorHandler(Error, "max_read_channel_block (Receive)");
	else
	{
		Text.Format("Data (%d Byte) = ", size);
		for (i=0; i<size; i++)	
		{
			Line.Format(" %X", data[i]);
			Text = Text + Line;
		}
		AfxMessageBox(Text);
	}
}

