/*************************************************************************/
//
// FP_TEST
//
// Demonstrates usage of floating point calculation on the MAX-PC.
// This program downloads the realtime program ..\rt\float.exe.
//
// The file maxw32.lib (from the corresponding directory MSVC or BC)
// has to be added to the project
//
// (C)2003 SORCUS Computer GmbH
/*************************************************************************/

#include "windows.h"				// defines basic data types  
#include "max_lib.h"				// all definitions for the MAXW32
#include "stdio.h"				// for printf 
#include "conio.h"				// for getch 
#include "math.h"				

#define VERSION_SAMPLE	1
#define REV_SAMPLE		'A'

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
	USHORT	 usMsgSize = 100;
	char		 sErrMsg[200];

	max_get_error_message(Error, &usMsgSize, sErrMsg);
	printf ("\n\rError %X in %s (%s)", Error, sText, sErrMsg);
	_getch();
	max_clear_error();
}

/****************************************************************************/
int main ()
{
	USHORT	 usBoardNr	= 1;	// number given in the setup-tool
										// (see 'All SORCUS Boards' in the Windows control panel)
	USHORT		usCPUSlot, usCPULayer;	// slot and layer of the X-MAX-1
	MAX_ERROR	Error;
	MAXMODHND	hModul;
	MAX_OS_INFO	rcOsInfo;
	MAX_VERSION	Version;
	ULONG			ulDate; 	
	char			szVersion[12], szDate[12], szOsX[12];
	char			filename[80];
	MAX_PGM_TYPE rcPrgInfo;
	USHORT		usTask = 0;
	double		dArgument, dResult[2], dControl1, dControl2;
	USHORT		usSize;
	
	printf("\n\rMAX-Sample01 - Version %d.%c\n\r", VERSION_SAMPLE, REV_SAMPLE);
	printf("\n\rInitialize a X-MAX-1 module...\n\r");

	// Init library
	Error = max_init_lib (0);

	// check if library is initialized successfully
	if (Error != ERR_OK) 
	{
		ErrorHandler(Error, "max_init_lib");
		return 1;
	}
	max_get_lib_version (&Version, &ulDate);
	max_get_version_string(Version, ulDate, szVersion, szDate);

	// get version information
	printf("\n\rMAXW32.DLL-Library Version: %s (%s)", szVersion, szDate);
	Error = max_get_drv_version (&Version, &ulDate);
	if (Error) 
		ErrorHandler(Error, "max_get_drv_version");
	else 
	{
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf("\n\rMAXNT4.SYS-Driver  Version: %s (%s)", szVersion, szDate);
	}
	
	// max_reset_board can be called also at any time but
	// it is only required if max_board_reacting returns an error
	Error = max_reset_board (usBoardNr, 10);
	if (Error) 
	{
		ErrorHandler(Error, "max_reset_board");
		return 1;
	}

	// connect program to the X-MAX-1 and get a handle for the module
	// if only one CPU module is plugged in MAX_AUTO_SLOT will cause the
	// max_connect_cpu function to find that CPU module automatically
	Error = max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, &rcOsInfo, &hModul);
	if (Error) 
	{
		// if connection fails try to reset the board again
		Error = max_reset_board (usBoardNr, 20);
		if (Error) 
		{
			ErrorHandler(Error, "max_reset_board");
			return 1;
		}
		else 
		{	
			Error = max_connect_cpu (usBoardNr, MAX_AUTO_SLOT, 0, &rcOsInfo, &hModul);
			if (Error) 
			{
				ErrorHandler(Error, "max_connect_cpu");
				return 1;
			}
		}
	}

	// if the basic MINI-OSX is the active operating system,
	// load the full ROM-OsX 
	if (rcOsInfo.usType == MAX_MINI_OSX) 
	{
		Error = max_load_osx (hModul, "ROM");
		if (Error) 
		{
			ErrorHandler(Error, "max_load_osx");
			return 1;
		}
	}
	// OsX Version
	Error = max_get_os_version(hModul, &rcOsInfo);
	if (Error) 
	{
		ErrorHandler(Error, "max_get_os_version");
		return 1;
	}

	// Location of the CPU module
	Error = max_get_slot(hModul, &usCPUSlot, &usCPULayer);
	if (Error) 
		ErrorHandler(Error, "max_get_slot");
	max_get_version_string(rcOsInfo.Version, rcOsInfo.ulDate, szVersion, szDate);
	switch (rcOsInfo.usType) 
	{
		case MAX_MINI_OSX: strcpy (szOsX, "Mini-OsX");
								 break;
		case MAX_ROM_OSX : strcpy (szOsX, "ROM-OsX");
								 break;
		case MAX_RAM_OSX : strcpy (szOsX, "RAM-OsX");
								 break;
	}
	printf("\n\rConnected to CPU on slot %X, layer %X: %s, Version=%s (%s)",
			  usCPUSlot, usCPULayer, szOsX, szVersion, szDate);

	printf("\n\r\n\rDownload a Real-Time-Program onto the module");
	// download RT-program on the X-MAX-1
	strcpy (filename, "..\\rt\\float.exe");
	rcPrgInfo.usNo			= 0x900C;
	rcPrgInfo.szName		= filename;
	rcPrgInfo.usTaskType = 0;	// parameter is taken from the PDT
	rcPrgInfo.usIrq		= 0;	// parameter is taken from the PDT
	rcPrgInfo.usTask		= 50;	
	rcPrgInfo.ulDataSize = 0;	// no data range
	rcPrgInfo.usFlags		= MAX_TASKNO_BY_USER + // use the given task number
								  MAX_CALL_AUTO_INIT;  // call procedure 1	
	rcPrgInfo.usLevel		= 3;	
	rcPrgInfo.usUsage		= 0;
 	Error = max_transfer_and_install(hModul, &rcPrgInfo, &usTask);
	if (Error) 
	{
		ErrorHandler(Error, "max_transfer_and_install");
		return 1;
	}
	
	// version of the RT program
	Error = max_get_prog_version(hModul, usTask, &Version, &ulDate);
	if (Error)
		ErrorHandler(Error, "max_get_prog_version");
	else 
	{
		max_get_version_string(Version, ulDate, szVersion, szDate);
		printf("\n\rProgram %d successfully installed as Task %X, Version %s", 
			    rcPrgInfo.usNo, usTask, szVersion);
	}
	// 1. Write argument value to OsX task
	// 2. Activate the task to calculate the result 
	//    and wait a little (floating point emulation is slow on the MAX-PC)
	
	// transfer floating point values as block (a double value is 8 byte long)
	// or ULONG (a float value is 4 byte long) to OsX tasks
	dArgument = 0.5;
	usSize = sizeof(double);
	Error = max_write_par_block(hModul, usTask, 0, &usSize, &dArgument); 
	if (Error) 
		ErrorHandler(Error, "max_write_par_block");
	else 
	{
		Error = max_wakeup_ni_task(hModul, usTask, 1);
		if (Error) 
			ErrorHandler(Error, "max_wakeup_ni_task");
		Sleep(50);
	}

	// 3. Read the results from the task and compare it
	usSize = 2 * sizeof(double);
	Error  = max_read_par_block(hModul, usTask, sizeof(double), &usSize, &dResult); 
	
	dControl1 = dArgument * 2;
	dControl2 = sin(dArgument);
	if ((dControl1 == dResult[0]) && (dControl2 == dResult[1]))
		printf("\n\rSuccess!");
	else
		printf("\n\rFP-Error");
	// exit library
	max_exit_lib();
	_getch();
	return 0;
}