/****************************************************************************/
// Sample Real Time program
//
// The program shows how to use floating point arithmetics under OsX.
// It does not do anything sensible - just the project settings
// and the handling of floating point operations is demonstrated.
// With RTDS MATHL.LIB and EMU.LIB have to be added to the project.
// From the Borland IDE MATHL.LIB is included implicitly.
// EMU.LIB must be used from the SORCUS libraries.
//
// Program Number 0x900C
//
// (C)2003 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                        // must be defined in every real time
                                       // program before including max_lib.h
#include "max_lib.h"							// SORCUS header
#include "dos.h"
#include "math.h"								// must be included for floating point operations

#define PROGRAMM_NUMBER 0x900C
#define VERSION         '1'
#define REVISION        'A'

void CalculateFloatingPoint(void);

// PDT structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
} rcPDT;

// parameter range structure
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
// offset 0
   double       dArgument;
   double       dResult1;
   double       dResult2;              // double is 8 byte long (float is 4 byte)
   MAX_ERROR    Error;                 // Errorcode
} parameter;

MAXMODHND    hMyself;               	// Handle for the CPU module

/****************************************************************************/
// Do floatig point operations always in a subroutine -
// never directly in a task function or procedure between
// max_entry_... and max_exit_...
// Note: x = 3.14; is also a floatig point operation!
void CalculateFloatingPoint()
{
	parameter.dArgument = 0.5;
   parameter.dResult1 = parameter.dArgument * 2;
   parameter.dResult2 = sin(parameter.dArgument);
}

/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT MainProc(void)
{
	USHORT usTask;
   usTask = max_entry_proc();
   CalculateFloatingPoint();
   max_sleep_task(hMyself, usTask);
   max_exit_proc();
}

/****************************************************************************/
// Init task parameter
void MAXEXPORT AutoInit(void)
{
   MAX_OS_INFO  rcOsX;
   max_entry_proc();
   parameter.Error = max_init_lib(0);
   // get a handle for the X-MAX-1
   parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0, &rcOsX, &hMyself);
   max_exit_proc();
};

/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_NI_TASK +
                                  MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = 0;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = 1;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;

   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}

