/****************************************************************************/
// Sample Real Time program to show how to use the graphic library
// for OsX on the X-KIT-3
//
// Program Number 900Bh
//
// (C)2003 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                 	// must be defined in every real time
// program before including max_lib.h
#include "max_lib.h"
#include "dos.h"
#include "stdlib.h"
#include "osxgraph.h"
#include "string.h"

#define PROGRAMM_NUMBER 0x900B
#define VERSION         '1'		//version of program
#define REVISION        'A'		//revision of program

// PDT structure
struct pdt_type
{
	MAX_PDT_HEAD    rcHead;
	void* MAXEXPORT main_proc;
	void* MAXEXPORT auto_init;
} rcPDT;

// parameter range structure
struct parameter_type
{
	MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
	// offset 0
	MAX_ERROR    Error;                 // Errorcode
	MAXMODHND    hMyself;               // Handle for the CPU module
	MAX_OS_INFO  rcOsX;                 // Version info about OsX
	USHORT       usCounter;
} parameter;

/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
void MAXEXPORT MainProc(void)
{
	UCHAR aucText[50];
	USHORT usX, usY, usSize, usX1, usY1, usX2, usY2, i, k, usXStart, usYStart, l, m, usScale;

	max_entry_proc();

	parameter.usCounter++;
	if (parameter.usCounter == 10000 || parameter.usCounter == 50000) 
	{	// clear screen
		max_set_pen_color(parameter.hMyself, 0, 0, 0); // black
		max_fill_rectangle(parameter.hMyself, 100, 70, 320, 220);
	}
	else
	{
		// change display every 30000 calls of this procedure
		if (parameter.usCounter == 30000)
		{
			// write "SORCUS" yellow
			max_set_pen_color(parameter.hMyself, 255, 255, 0);
			max_set_font(parameter.hMyself, FONT_MONOSPACED, FONT_STYLE_PLAIN, FONT_16_24);
			strcpy(aucText, "SORCUS");
			usSize = strlen(aucText);
			max_draw_text(parameter.hMyself, 100, 100, aucText, usSize);
		}
		else
		{
			if (parameter.usCounter == 60000)
			{	// plot big SORCUS logo
				// print yellow lines of the 'S'
				max_set_pen_color(parameter.hMyself, 255, 255, 0);
				usXStart = 100;
				usYStart = 70;
				usScale = 2;
				usY1 = usYStart;
				usY2 = usY1;
				usX1 = usXStart + 1 + 13*usScale;
				usX2 = usXStart + 13*usScale + 27*usScale;
				for (i=0; i<13*usScale; i++)
				{
					max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
					usX1--;
					usY1++;
					usY2 = usY1;
				}
	
				// print yellow and green lines of the 'S'
				usX1 = usXStart + 1;
				usX2 = usXStart + 13*usScale;
				max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
				usY1++;
				usY2 = usY1;
				for (i=0; i<13*usScale; i++)
				{
					max_set_pen_color(parameter.hMyself, 255, 255, 0);
					usX1 = usXStart + 1;
					usX2 = usXStart + 13*usScale;
					max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
					max_set_pen_color(parameter.hMyself, 0, 255, 0);
					usX1 = usXStart + 2 + 13*usScale;
					usX2 = usXStart + 13*usScale + 27*usScale + 1 + i;
					max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
					usY1++;
					usY2 = usY1;
				}
			
				usX1 = usXStart + 1;
				usX2 = usXStart + 13*usScale;
				max_set_pen_color(parameter.hMyself, 255, 255, 0);
				max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
			
				usX1 = usXStart + 13*usScale + 27*usScale + 2;
				usX2 = usXStart + 13*usScale + 27*usScale + 13*usScale + 1;
				max_set_pen_color(parameter.hMyself, 0, 255, 0);
				max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
			
				usY1++;
				usY2 = usY1;
				for (i=0; i<13*usScale; i++)
				{
					max_set_pen_color(parameter.hMyself, 255, 255, 0);
					usX1 = usXStart + 2 + i;
					usX2 = usXStart + 13*usScale + 27*usScale;
					max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
					max_set_pen_color(parameter.hMyself, 0, 255, 0);
					usX1 = usXStart + 13*usScale + 27*usScale + 2;
					usX2 = usXStart + 13*usScale + 27*usScale + 13*usScale + 1;
					max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
					usY1++;
					usY2 = usY1;
				}
	
				// print green lines of the 'S'
				usX1 = usXStart + 2 + 13*usScale + 27*usScale;
				usX2 = usXStart + 1 + 13*usScale + 27*usScale + 13*usScale;
				max_set_pen_color(parameter.hMyself, 0, 255, 0);
				max_draw_line(parameter.hMyself, usX1, usY1, usX2, usY2);
	
				usY1++;
				usY2 = usY1;
				usX1 = 2 + usXStart + 13*usScale;
				usX2 = usXStart + 13*usScale + 27*usScale + 13*usScale;
				max_set_pen_color(parameter.hMyself, 0, 255, 0);
				for (i=0; i<13*usScale; i++)
				{
					max_draw_line(parameter.hMyself, usX1, usY1, usX2-i, usY2);
					usY1++;
					usY2 = usY1;
				}

				parameter.usCounter = 0;
			}
		}
	}
	max_exit_proc();
}

/****************************************************************************/
// Init task parameter
void MAXEXPORT AutoInit(void)
{
	USHORT usTask, usCacheMode;

	usTask = max_entry_proc();
	max_init_lib(0);
	parameter.usCounter = 0;
	// get a handle for the X-MAX-1
	parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,&parameter.rcOsX, &parameter.hMyself);

	// set chache mode 
	usCacheMode = 2;
	parameter.Error = max_cache_control(parameter.hMyself, &usCacheMode);

	// init graphic
	parameter.Error = max_init_graphic(parameter.hMyself, HITACHI_SX16H003ZZA_DISPLAY, GRAPHIC_MODE, 0);	

	// activate the task
	parameter.Error = max_wakeup_ni_task(parameter.hMyself, usTask, 1);
	max_exit_proc();
};



/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
	// the PDT-flags determine the task-type and
	// information how the data and parameter range are allocated
	
	//set flags for task type
	rcPDT.rcHead.usFlags         = MAX_NI_TASK +	MAXPDT_PARAMETER_IN_PROG;
	
	// interrupt source
	rcPDT.rcHead.usInterrupt     = 0;
	
	// the program number identifies the program uniquely
	rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;
	
	// number of global task procedures in the PDT
	rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;
	
	// the following lines are just for documentation and information purpose
	rcPDT.rcHead.ucType          = 1;   // always !
	rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
	rcPDT.rcHead.cVersion        = VERSION;
	rcPDT.rcHead.cRevision       = REVISION;
	rcPDT.rcHead.ucCPU           = MAX_CPU486;
	rcPDT.rcHead.ucCoProc        = _NOCOPROC;
	rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
	rcPDT.rcHead.ucProgType      = 0;   // not used
	rcPDT.rcHead.ulRes1          = 0;
	rcPDT.rcHead.ulRes2          = 0;
	rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet
	
	// address and size of the tasks data range
	// only when MAXPDT_DATA_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulDataAdr       = 0;
	rcPDT.rcHead.ulDataSize      = 0;
	
	// address and size of the tasks parameter range
	// only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
	rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);
	
	// initialize global procedure addresses in PDT
	rcPDT.main_proc        = MainProc;
	rcPDT.auto_init        = AutoInit;
	//ToDo: register user defined procedures:
	
	// transfer PDT information to the OsX
	max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}

