#include "stdafx.h"
#include "Measure.H"



/******************************************************************************
*                   CMeasurement::CMeasurement
*   PUPROSE
*   REMARK
*   PARAMETER
*   RETURN
*   HISTORY
******************************************************************************/
CMeasurement::CMeasurement(ULONG ulBoardNumber, ULONG ulXmax1Slot, ULONG ulXad1420Slot, ULONG ulNumberOfData, double dFrequency)
{
    // set board information
    ////////////////////////////////////////////////////
    m_rcBoard.usNumber = ulBoardNumber;
    m_rcBoard.usTimeout = 10;

    // set X-MAX-1 information
    ////////////////////////////////////////////////////
    m_rcXmax1.usSlot = ulXmax1Slot;
    m_rcXmax1.usLayer = 0;

    // set X-AD14-20 information
    ////////////////////////////////////////////////////
    m_rcXad1420.usSlot = ulXad1420Slot;
    m_rcXad1420.usLayer = 0;
   
    // set measurement information
    ////////////////////////////////////////////////////
    m_ulNumberOfData = ulNumberOfData;
    m_dFrequency = dFrequency;

    // set data for rt programm installation
    ////////////////////////////////////////////////////
    strcpy(m_acRtFileName, "ExDac2.exe");
    m_rcRtProgram.szName = m_acRtFileName;
    m_rcRtProgram.usNo = EXDAC_PROG_NO;
    m_rcRtProgram.usUsage = 0;
    m_rcRtProgram.usTask = 111;
    m_rcRtProgram.usTaskType = MAX_II_TASK;
    m_rcRtProgram.usLevel = 0;
    m_rcRtProgram.usIrq = MAX1_IRQ_TIMERA;
    m_rcRtProgram.usFlags = MAX_CALL_AUTO_INIT | MAX_TASKTYPE_BY_INSTALL | MAX_TASKNO_BY_INSTALL;
    m_rcRtProgram.ulDataSize = 0;
}



/******************************************************************************
*								CMeasurement::~CMeasurement
*	PURPOSE
*	REMARK
*	PARAMETER
*	RETURN
*	HISTORY
******************************************************************************/
CMeasurement::~CMeasurement()
{
    TRACE("CMeasurement::~CMeasurement()\r\n");

    // delete allocated PC buffer for AIN
    free(m_arcPcAinBuffer[0].prcStart);
    free(m_arcPcAinBuffer[1].prcStart);
}



/******************************************************************************
*                   CMeasurement::Init
*   PUPROSE
*   REMARK
*   PARAMETER
*   RETURN
*   HISTORY
******************************************************************************/
ULONG CMeasurement::Init()
{
    MAX_ERROR Error = ERR_OK;
    MAX_BUFFER_TYPE rcBufferType;
    ULONG i;
    DAQ_INIT_DATA rcInitData;
    USHORT usInsize, usOutsize, usTask;;



    // reset board
    ////////////////////////////////////////////////////
    Error = max_reset_board(m_rcBoard.usNumber, m_rcBoard.usTimeout);
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_reset_board", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    // connect X-MAX-1
    ////////////////////////////////////////////////////
    Error = max_connect_cpu(m_rcBoard.usNumber,
                            m_rcXmax1.usSlot,
                            m_rcXmax1.usLayer,
                            &m_rcXmax1.rcOsInfo,
                            &m_rcXmax1.hModule);
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_connect_cpu", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    // load ROM OsX
    ////////////////////////////////////////////////////
    Error = max_load_osx(m_rcXmax1.hModule, "ROM");
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_load_osx", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    // load X-AD14-20 MDD
    ////////////////////////////////////////////////////
    Error = max_load_mdd(   m_rcXmax1.hModule,
                            m_rcXad1420.usSlot,
                            m_rcXad1420.usLayer,
                            0,
                            0x8024,
                            NULL,
                            &m_rcXad1420.hMdd);
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_load_mdd X-AD14-20", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    // set user service routine
    ////////////////////////////////////////////////////
    Error = max_set_service(m_rcXmax1.hModule, MeasurementService);
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_set_service", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    // install rt measurement task
    ////////////////////////////////////////////////////
    Error = max_transfer_and_install(m_rcXmax1.hModule, &m_rcRtProgram, &usTask);
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_transfer_and_install", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    // allocate OsX Buffer
    ////////////////////////////////////////////////////
    rcBufferType.usStrategy = MAX_ALLOC_DOWN_MAX;       // allocate as many as possible
    rcBufferType.usAlignment = 2;
    rcBufferType.usTask = 0;
    rcBufferType.usUsage = 0;
    m_rcOsXBuffer.ulSize = 100000*sizeof(OSX_BUFFER_DATA);
    Error = max_create_buffer(m_rcXmax1.hModule, &rcBufferType, &m_rcOsXBuffer.ulSize, &m_rcOsXBuffer.hBufHandle);
    if (Error != ERR_OK)
    {
        MessageBox(NULL, "Error max_create_buffer", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }


    // allocate PC data buffer for AIN-x channel
    ////////////////////////////////////////////////////
    for (i=0; i<2; i++)
    {
        m_arcPcAinBuffer[i].ulSize = m_ulNumberOfData;
        m_arcPcAinBuffer[i].ulFree = m_arcPcAinBuffer[i].ulSize;
        m_arcPcAinBuffer[i].ulIndex = 0;
        m_arcPcAinBuffer[i].ulError = NO_ERROR;
        m_arcPcAinBuffer[i].prcStart = (PC_AIN_BUFFER_DATA*)malloc(m_arcPcAinBuffer[i].ulSize*sizeof(PC_AIN_BUFFER_DATA));
        m_arcPcAinBuffer[i].prcEnd = m_arcPcAinBuffer[i].prcStart + (m_arcPcAinBuffer[i].ulSize*sizeof(PC_AIN_BUFFER_DATA));
        m_arcPcAinBuffer[i].prcRead = m_arcPcAinBuffer[i].prcStart;
        m_arcPcAinBuffer[i].prcWrite = m_arcPcAinBuffer[i].prcStart;
    }

    // set parameter of rt measurement task
    ////////////////////////////////////////////////////
    rcInitData.ulAcquisitions = m_ulNumberOfData;
    Error = max_get_buffer_id(m_rcOsXBuffer.hBufHandle, &rcInitData.ulBufferID);
    rcInitData.usADSlot = m_rcXad1420.usSlot;

    // check frequency
    if ( (m_dFrequency < 20) || (m_dFrequency > 1189200) )
    {
        MessageBox(NULL, "Error wrong frequency (20 ... 1189200)", "Init", MB_ICONERROR);
        return ERR_EXDAC;
    }

    rcInitData.usTimeValue = (USHORT)(1189200.0 / m_dFrequency);
    usInsize = sizeof(DAQ_INIT_DATA);
    usOutsize = 0;
    Error = max_call_func(m_rcXmax1.hModule, m_rcRtProgram.usTask, 2, &usInsize, (void*)&rcInitData, &usOutsize, NULL);

    // start measurement
    Error = max_call_proc(m_rcXmax1.hModule, m_rcRtProgram.usTask, 3);

    return 0;
}
