#include "windows.h"
#include "max_lib.h"
#include "x_mdd.h"
#include "..\exdacifc.h"


///////////////////////////////////////////////////////////
// struct
///////////////////////////////////////////////////////////

// data within PC AIN buffer
///////////////////////////////////////
struct PC_AIN_BUFFER_DATA
{
    ULONG       ulIndex;        // data index
    LONG        lData;          // data
};

// PC AIN buffer information
///////////////////////////////////////
struct PC_AIN_BUFFER
{
    PC_AIN_BUFFER_DATA*     prcStart;       // pointer to buffer start
    PC_AIN_BUFFER_DATA*     prcEnd;         // pointer to buffer end
    ULONG                   ulSize;         // size of buffer
    ULONG                   ulFree;         // free buffer space
    ULONG                   ulIndex;        // data index
    PC_AIN_BUFFER_DATA*     prcRead;        // read pointer
    PC_AIN_BUFFER_DATA*     prcWrite;       // write pointer
    ULONG                   ulError;        // error information
};


// OsX buffer information
///////////////////////////////////////
struct OSX_BUFFER
{
    MAXBUFHND   hBufHandle;                 // buffer handle
    ULONG       ulSize;                     // buffer size
};


// X-Bus board information
///////////////////////////////////////
struct BOARD
{
    USHORT      usNumber;                   // number of board
    USHORT      usTimeout;                  // timeout value
};

// X-MAX-1 information
///////////////////////////////////////
struct XMAX1
{
    USHORT      usSlot;                     // slot
    USHORT      usLayer;                    // layer
    MAX_OS_INFO rcOsInfo;                   // OsX information
    MAXMODHND   hModule;                    // handle to module
};

// X-AD14-20 information
///////////////////////////////////////
struct XAD1420
{
    USHORT      usSlot;                     // slot
    USHORT      usLayer;                    // layer
    MAXMDDHND   hMdd;                       // handle to MDD
};


///////////////////////////////////////////////////////////
// class
///////////////////////////////////////////////////////////

/******************************************************************************
*                   class CMeasurement
*   PUPROSE
*   REMARK
*   PARAMETER
*   RETURN
*   HISTORY
******************************************************************************/
class CMeasurement
{
public:
    CMeasurement(ULONG ulBoardNumber, ULONG ulXmax1Slot, ULONG ulXad1420Slot, ULONG ulNumberOfData, double dFrequency);      // constructor
    ~CMeasurement();                                            // destructor
    ULONG Init();                                               // init measurement

    // board and module information
    BOARD               m_rcBoard;                              // base board information
    XMAX1               m_rcXmax1;                              // X-MAX-1 information
    XAD1420             m_rcXad1420;                            // X-AD14-20 information

    ULONG               m_ulNumberOfData;                       // number of data
    double              m_dFrequency;                           // frequency

    // buffer information
    OSX_BUFFER          m_rcOsXBuffer;                          // OsX buffer information
    PC_AIN_BUFFER       m_arcPcAinBuffer[2];                    // PC buffer for AIN data

    // rt program information
    MAX_PGM_TYPE        m_rcRtProgram;                          // rt program information
    CHAR                m_acRtFileName[1000];                   // rt program file name
};




///////////////////////////////////////////////////////////
// function
///////////////////////////////////////////////////////////

// service routine called by real time task
MAX_CALLBACK MeasurementService(MAXMODHND hModul, ULONG ulMessage);
