//****************************************************************************************************
// This program uses the RT-program osxprn.exe to print "Hello World!" to a printer connected to
// a X-MAX-1 module.
// It demonstrates how to load a RT-program and access subroutines of it.
// On startup the function max_rtprint_init is called to install the RT-program. The function
// max_rtprint_line gives then access to the RT-program's subroutine, which will send a given string
// to the parallel port of the X-MAX-1 module.
// 
// Please set the card, slot, layer parameters according to your setup in the main() function
//
// (c) 2001 by SORCUS Computer GmbH
//****************************************************************************************************

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "max_lib.h"



//variables to handle RT program
MAX_PGM_TYPE program;
USHORT task;


//load RT program and call auto_init
MAX_ERROR max_rtprint_init(MAXMODHND hCPUModule)
{
	MAX_ERROR Error;
	char name[80];


	//set task number
	program.usTask=100;
	//give the name of the .EXE
	strcpy(name,"..\\rt\\osxprn.exe");
	program.szName=name;
	//ProgNo
	program.usNo = 0xa006;
	//task type
	program.usTaskType=MAX_NI_TASK;
	//priority
	program.usLevel=1;
	//call auto_init
	program.usFlags=MAX_CALL_AUTO_INIT|MAX_TASKNO_BY_USER;
	//must be zero
	program.usUsage=0;

	//install program
	Error = max_transfer_and_install(hCPUModule,&program,&task);

	return(Error);
}


//call line printing routine of RT program
MAX_ERROR max_rtprint_line(MAXMODHND hCPUModule, CHAR *line, USHORT length)
{
	MAX_ERROR Error;
	USHORT outsize=0;

	Error = max_call_func(hCPUModule, task, 2, &length, line, &outsize, NULL);

	return(Error);
}


int main()
{
	MAX_ERROR Error;
	CHAR acError[80]; //string for error messages
	USHORT nELength=80; //length of error string
	MAXMODHND hCPUModule;
	MAX_OS_INFO os;
	CHAR line[256];
	int card,slot,layer;

	//////////////////////////////////////////
	//set this according to your configuration
	card=1;
	slot=1;
	layer=0;
	//////////////////////////////////////////

	max_init_lib(0);

	//reset board
	Error = max_reset_board(card,100);
	if (Error!=0)
	{
		max_get_error_message(Error,&nELength,acError);
		printf("max_reset_board: %s\n",acError);
	}
 
	//connect to the CPU module
	Error = max_connect_cpu(card,slot,layer,&os,&hCPUModule);
	if (Error!=0)
	{
		max_get_error_message(Error,&nELength,acError);
		printf("max_connect_cpu: %s\n",acError);
	}

	//load OsX
	Error = max_load_osx(hCPUModule,"ROM\0");
	if (Error!=0)
	{
		max_get_error_message(Error,&nELength,acError);
		printf("max_load_osx: %s\n",acError);
	}

	//init printing support
	Error = max_rtprint_init(hCPUModule);
	if (Error!=0)
	{
		max_get_error_message(Error,&nELength,acError);
		printf("max_rtprint_init: %s\n",acError);
	}

	//print a line

	strcpy(line,"Hello World!");
	//append a form feed signal to eject paper
	line[12]=0x0c;
	//call subroutine (length of string to print is 13)
	Error = max_rtprint_line(hCPUModule,line,13);
	if (Error!=0)
	{
		max_get_error_message(Error,&nELength,acError);
		printf("max_rtprint_line: %s\n",acError);
	}

	//close channel to MAX-1
	max_exit_cpu(&hCPUModule);

	max_exit_lib();

 	return 0;
}
