VERSION 5.00
Begin VB.Form Init 
   Caption         =   "Sample program for MAX6pci: Init Board"
   ClientHeight    =   3240
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   7545
   FillColor       =   &H00C0FFFF&
   LinkTopic       =   "Form1"
   ScaleHeight     =   3240
   ScaleWidth      =   7545
   StartUpPosition =   3  'Windows-Standard
   Begin VB.CommandButton StartStop 
      Caption         =   "Start/Stop"
      Enabled         =   0   'False
      Height          =   855
      Left            =   120
      TabIndex        =   3
      Top             =   2160
      Width           =   1695
   End
   Begin VB.CommandButton Download 
      Caption         =   "Download"
      Enabled         =   0   'False
      Height          =   735
      Left            =   120
      TabIndex        =   2
      Top             =   1200
      Width           =   1695
   End
   Begin VB.TextBox InfoBox 
      Enabled         =   0   'False
      Height          =   2775
      Left            =   2040
      MultiLine       =   -1  'True
      TabIndex        =   1
      Top             =   240
      Width           =   5175
   End
   Begin VB.CommandButton Start 
      Caption         =   "Start"
      Height          =   735
      Left            =   120
      TabIndex        =   0
      Top             =   240
      Width           =   1695
   End
End
Attribute VB_Name = "Init"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'*************************************************************************/
'
'  SAMPLE01
'
'  Demonstrates program initialization of a PC program that uses MAX6pci.
'  Download a real-time program onto a X-MAX-1 that lets the LED blink.
'  Set some parameter in the real-time program.
'
'  Copyright SORCUS Computer GmbH 2001
'*************************************************************************/
Option Explicit

Dim hModul As Long
Dim usTask As Integer
Dim usState As Integer

' internal helper function to display error messages
Private Sub ErrorHandler(ByVal Error As Integer, ByVal source As String)
    Dim usMsgSize As Integer
    Dim sMsgText As String
    Dim sTitle As String
    Dim sErrMsg As String * 100 ' provide enough memory space
                                ' where max_get_error_message can write to
    usMsgSize = 100             ' max size of message text
    Call max_get_error_message(Error, usMsgSize, sErrMsg)
    sMsgText = "Error " & Format(Hex(Error), "0000") & "h : " & sErrMsg
    sTitle = "Error calling function " & source
    Error = MsgBox(sMsgText, vbExclamation, sTitle)
    Call max_clear_error
    
End Sub

Private Sub Start_Click()

    Dim usBoardNr As Integer        ' number given in the setup-tool
                                    ' (see 'All SORCUS Boards' in the Windows control panel)
    Dim usCPUSlot   As Integer      ' slot of the X-MAX-1
                                    ' (see printed number on the carrier board)
    Dim usCPULayer  As Integer      ' layer where the X-MAX-1 is
                                    ' (directly on the MAX6 carrier board is 0)
    Dim Error As Integer
    Dim Version As Long
    Dim ulDate As Long
    Dim szVersion As String * 16
    Dim szDate As String * 16
    Dim rcOsInfo As MAX_OS_INFO
    Dim rcModulInfo As MAX_MODULINFO_TYPE
    Dim rcBoardInfo As MAX_BOARD_TYPE
    Dim usSlot As Integer
    Dim ulSerNo As Long
    
    usBoardNr = 1
    usState = 0

    ' Init library
    Error = max_init_lib(0)
    
    ' check if library is initialized successfully
    If Error <> ERR_OK Then
       Call ErrorHandler(Error, "max_init_lib")
       Exit Sub
    End If
    Call max_get_lib_version(Version, ulDate)
    Call max_get_version_string(Version, ulDate, szVersion, szDate)

    ' get version information about the DLL and the driver
    InfoBox.Text = "MAXW32.DLL-Library Version: " & szVersion
    InfoBox.Text = InfoBox.Text & ", " & szDate
    InfoBox.Text = InfoBox.Text & vbCrLf
    Error = max_get_drv_version(Version, ulDate)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_get_drv_version")
    Else
        Call max_get_version_string(Version, ulDate, szVersion, szDate)
        InfoBox.Text = InfoBox.Text & "MAXNT4.SYS-Driver Version: " & szVersion
        InfoBox.Text = InfoBox.Text & ", " & szDate
        InfoBox.Text = InfoBox.Text & vbCrLf
    End If
    InfoBox.Text = InfoBox.Text & vbCrLf

    ' Check the carrier board
    Error = max_get_board_info(usBoardNr, rcBoardInfo)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_get_board_info")
        Exit Sub
    End If
    InfoBox.Text = InfoBox.Text & "Board " & usBoardNr
    InfoBox.Text = InfoBox.Text & " : Type: " & rcBoardInfo.usType
    InfoBox.Text = InfoBox.Text & ", Version: " & rcBoardInfo.usVersion
    ulSerNo = ConvertMaxLongToLong(rcBoardInfo.ulSerNr)
    InfoBox.Text = InfoBox.Text & ", Serial-No: " & ulSerNo
    InfoBox.Text = InfoBox.Text & vbCrLf
    InfoBox.Text = InfoBox.Text & vbCrLf
    
    ' max_reset_board can be called also at any time but
    ' it is only required if max_board_reacting returns an error
    Error = max_reset_board(usBoardNr, 10)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_reset_board")
        Exit Sub
    End If
    
    ' connect program to the X-MAX-1 and get a handle for the module
    ' if only one CPU module is plugged in MAX_AUTO_SLOT will cause the
    ' max_connect_cpu function to find that CPU module automatically
    Error = max_connect_cpu(usBoardNr, MAX_AUTO_SLOT, 0, rcOsInfo, hModul)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_connect_cpu")
        Exit Sub
    End If

    ' if the basic MINI-OSX is the active operating system,
    ' activate the full ROM-OsX or load a RAM-OsX from a file
    If rcOsInfo.usType = MAX_MINI_OSX Then
        ' with MAX1_NEWEST_OSX it automatically
        ' searches for the newest OsX-file first in the actual directory
        ' the in the SNW32\OSX directory
'       Error = max_load_osx (hModul, MAX1_NEWEST_OSX)
        Error = max_load_osx(hModul, "ROM")
        If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_load_osx")
            Exit Sub
        End If
    End If
    
    ' OsX Version
    Error = max_get_os_version(hModul, rcOsInfo)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_get_os_version")
        Exit Sub
    End If

    ' Location of the CPU module
    Error = max_get_slot(hModul, usCPUSlot, usCPULayer)
    If Error <> ERR_OK Then
       Call ErrorHandler(Error, "max_get_slot")
    End If
    
    ' because visual basic is not able to make a correct alignment
    ' the definition of the type MAX_OS_INFO is different from the manual:
    ' instead of 'long' the type MAX_LONG is used.
    ' A conversion of that elements is neccessary for calling max_get_version_string
    ulDate = ConvertMaxLongToLong(rcOsInfo.ulDate)
    Version = ConvertMaxLongToLong(rcOsInfo.Version)
    Call max_get_version_string(Version, ulDate, szVersion, szDate)
    If rcOsInfo.usType = MAX_ROM_OSX Then
       InfoBox.Text = InfoBox.Text & "CPU-Module with ROM-OsX Version: " & szVersion
    Else
       InfoBox.Text = InfoBox.Text & "CPU-Module with RAM-OsX Version: " & szVersion
    End If
    InfoBox.Text = InfoBox.Text & ", " & szDate
    InfoBox.Text = InfoBox.Text & vbCrLf
    InfoBox.Text = InfoBox.Text & vbCrLf
    
    InfoBox.Text = InfoBox.Text & "Available Modules:" & vbCrLf
    For usSlot = 1 To 6
        Error = max_get_module_info(usBoardNr, usSlot, 0, rcModulInfo)
        If Error = ERR_OK Then
            InfoBox.Text = InfoBox.Text & " Slot " & usSlot
            InfoBox.Text = InfoBox.Text & " : Type " & rcModulInfo.usType
            InfoBox.Text = InfoBox.Text & ", Variant " & rcModulInfo.usVariant & vbCrLf
        Else
            If Error <> ERR_MODULE_NOT_AVAILABLE Then
                Call ErrorHandler(Error, "max_get_module_info")
                Exit For
            End If
        End If
        Next usSlot
 
    Download.Enabled = True
    Start.Enabled = False
    
End Sub

Private Sub Download_Click()

    Dim Error As Integer
    Dim Version As Long
    Dim ulDate As Long
    Dim szVersion As String * 16
    Dim szDate As String * 16
    Dim rcPrgInfo As MAX_PGM_TYPE

    ' download RT-program LED_II on the X-MAX-1
    rcPrgInfo.usNo = &H9001
    rcPrgInfo.szName = "..\..\rt\led_ii\c\led_ii.exe"
    rcPrgInfo.usTaskType = 0        ' parameter is taken from the PDT
    rcPrgInfo.usIrq = 0             ' parameter is taken from the PDT
    rcPrgInfo.usTask = 50           ' number of the task
    rcPrgInfo.ulDataSize = 0        ' no data range
    rcPrgInfo.usLevel = 3
    rcPrgInfo.usUsage = 0
    rcPrgInfo.usFlags = MAX_TASKNO_BY_USER + MAX_CALL_AUTO_INIT
    Error = max_transfer_and_install(hModul, rcPrgInfo, usTask)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_transfer_and_install")
        Exit Sub
    End If

    ' version of the RT program
    Error = max_get_prog_version(hModul, usTask, Version, ulDate)
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_get_prog_version")
    Else
        Call max_get_version_string(Version, ulDate, szVersion, szDate)
        InfoBox.Text = InfoBox.Text & "Program successfully downloaded, Version " & szVersion
        InfoBox.Text = InfoBox.Text & vbCrLf
    End If

    ' set LED blink frequency
    Error = max_write_par_ushort(hModul, usTask, 4, 20)
    If Error <> ERR_OK Then
       Call ErrorHandler(Error, "max_write_par_ushort")
    End If
    StartStop.Enabled = True
    Download.Enabled = False
   
 End Sub
 
Private Sub StartStop_Click()
    Dim Error As Integer
    
    If usState = 0 Then
        ' call start procedure (led starts blinking)
        Error = max_call_proc(hModul, usTask, 2)
        usState = 1
        StartStop.Caption = "Stop"
    Else
        ' call stop procedure (led stops blinking)
        Error = max_call_proc(hModul, usTask, 3)
        usState = 0
        StartStop.Caption = "Start"
    End If
    If Error <> ERR_OK Then
        Call ErrorHandler(Error, "max_call_proc")
        Exit Sub
    End If
End Sub


Private Sub Form_unload(Cancel As Integer)
    Dim Error As Integer
    Error = max_exit_lib
    If Error <> ERR_OK Then
       Call ErrorHandler(Error, "max_exit_lib")
    End If
End Sub

