/****************************************************************************/
/*                                                                          */
/* Sample for usage of the MDD for X-5B-1 modules                           */
/*                                                                          */
/* Demonstrates usage of all types of channel available for the module      */
/*                                                                          */
/* Tested with 5B02 panel with 5B41 and 5B39 signal conditioners            */
/*                                                                          */
/* Copyright SORCUS Computer GmbH 2004                                      */
/****************************************************************************/

#include "windows.h"
#include "stdio.h"
#include "conio.h"
#include "max_lib.h"
#include "x_mdd.h"

/****************************************************************************/
// Procedure to display and handle errors
void ErrorHandler(MAX_ERROR Error, char *sText)
{
    USHORT	 usMsgSize = 100;
    char		 sErrMsg[200];
    
    max_get_error_message(Error, &usMsgSize, sErrMsg);
    printf ("\n\rError %04Xh in %s (%s)", Error, sText, sErrMsg);
    //	_getch();									// user has to confirm
    max_clear_error();
}

/****************************************************************************/
void main () 
{
    MAX_ERROR		Error;
    MAX_OS_INFO		rcOsInfo;
    MAX_VERSION		Version;
    ULONG			ulDate;
    char			szVersion[12], szDate[12];
    CPS_X5B1		rcChannelCfg;
    MAXCHLHND		hAin, hAout, hTrigger; 	
    USHORT			usBoardNr;					
    USHORT			usADSlot, usADLayer;		
    MAXMODHND		hModul;
    MAXMDDHND		hMdd;
    LONG			lInput, lOutput;
    double			fpValue;
    
    usBoardNr = 1;									// see contol panel 'SORCUS boards'
    usADSlot  = 4;									// Location of the X-5B-1 module
    usADLayer = 0;
    
    printf ("Sample program : Data acquisition with X-5B-1");
    Error = max_init_lib (0);
    
    // display some information about the system
    max_get_lib_version (&Version, &ulDate);
    max_get_version_string(Version, ulDate, szVersion, szDate);
    printf ("\n\rLib Version: %s (%s)", szVersion, szDate);
    Error = max_get_drv_version (&Version, &ulDate);
    if (Error) 
    {
        printf ("Error %X", Error);
        max_clear_error();
    }
    else 
    {
        max_get_version_string(Version, ulDate, szVersion, szDate);
        printf ("\n\rDriver Version: %s (%s)", szVersion, szDate);
    }
    
    // reset the board
    Error = max_reset_board (usBoardNr, 20);
    if (Error) 
    {
        ErrorHandler(Error, "max_reset_board");
        max_exit_lib();
        return;
    }
    
    // get a cpu handle
    // if an X-MAX-1 should be used enter MAX_AUTO_SLOT for the slot number
    Error = max_connect_cpu (usBoardNr, 0, 0, &rcOsInfo, &hModul);
    if (Error) 
    {
        ErrorHandler(Error, "max_connect_cpu");
        max_exit_lib();
        return;
    }
    
    // load MDD for X-5B-1
    Error = max_load_mdd (hModul, usADSlot, usADLayer, 0, 
                          MDD_X_5B_1, NULL, &hMdd);
    if (Error) 
    {
        ErrorHandler(Error, "max_load_mdd");
        max_exit_lib();
        return;
    }
    // show info about used MDD
    max_get_mdd_version(hMdd, &Version, &ulDate);
    max_get_version_string(Version, ulDate, szVersion, szDate);
    printf ("\n\rMDD for X-5B-1 loaded, Version %s", szVersion);
    
    // open a channel to AIN on slot 12 of panel 0
    rcChannelCfg.usDevice	  = DEVICE_AIN_DIFF;
    rcChannelCfg.usIndexFirst = 12;
    rcChannelCfg.usIndexLast  = 12;
    rcChannelCfg.usFlags	  = _CP_UNCORRECTED;
    rcChannelCfg.usReadMode	  = IO_MODE_DIRECT;
    rcChannelCfg.usWriteMode  = 0;
    rcChannelCfg.usRange	  = RANGE_BIP_5V;
    rcChannelCfg.usOversampling = 1;
    rcChannelCfg.usSettleTime = 2;  // depends on the 5B signal conditioners bandwidth
    Error = max_open_channel (hMdd, sizeof(CPS_X5B1), &rcChannelCfg, NULL, 0, &hAin);
    if (Error) 
    {
        ErrorHandler(Error, "max_open_channel to AIN");
        max_exit_lib();
        return;
    }

    // open a channel to AOUT on slot 3 of panel 0
    rcChannelCfg.usDevice	  = DEVICE_AOUT;
    rcChannelCfg.usIndexFirst = 3;
    rcChannelCfg.usIndexLast  = 3;
    rcChannelCfg.usFlags	  = _CP_UNCORRECTED;
    rcChannelCfg.usReadMode	  = IO_MODE_RAM;
    rcChannelCfg.usWriteMode  = IO_MODE_DIRECT;
    rcChannelCfg.usRange	  = RANGE_BIP_5V;
    rcChannelCfg.usSettleTime = 2;  // depends on the 5B signal conditioners bandwidth
    Error = max_open_channel (hMdd, sizeof(CPS_X5B1), &rcChannelCfg, NULL, 0, &hAout);
    if (Error) 
    {
        ErrorHandler(Error, "max_open_channel to AOUT");
        max_exit_lib();
        return;
    }

    // Open a trigger channel for refresh of analog output values:
    // if the analog output has to be refreshed in shorter intervals
    // than new values are written to the output max_trigger_channel 
    // can be used with the handle retrievd from the following trigger Device
    // (this is faster than writing the same output value to the output channel
    // continuously)
    rcChannelCfg.usDevice	  = DEVICE_TRIGGER;
    rcChannelCfg.usIndexFirst = 3;
    rcChannelCfg.usIndexLast  = 3;
    rcChannelCfg.usFlags	  = 0;
    rcChannelCfg.usReadMode	  = 0;
    rcChannelCfg.usWriteMode  = 0;
    rcChannelCfg.usRange	  = RANGE_BIP_5V;
    rcChannelCfg.usSettleTime = 2;  // depends on the 5B signal conditioners bandwidth
    Error = max_open_channel (hMdd, sizeof(CPS_X5B1), &rcChannelCfg, NULL, 0, &hTrigger);
    if (Error) 
    {
        ErrorHandler(Error, "max_open_channel to AOUT");
        max_exit_lib();
        return;
    }

    /****************************************************************************/
    printf ("\n\rPress any key to start measurement");
    _getch();
    
    lOutput = 0;    // output start value = 0V
    // Read and write channels until a key is pressed...
    while (!_kbhit())
    {
        // Read analog input and display it
        Error = max_read_channel_long (hAin, &lInput);
        if (Error) 
        {
            ErrorHandler(Error, "max_read_channel_long");
            break;
        }
        printf("\n\r");
        fpValue = (double)lInput / 1000000;
        printf("Ain=%+.3fV  ", fpValue);

        // Write analog output (this should be used only if a new value should 
        // become active on the output, otherwise use max_trigger_channel)
        Error = max_write_channel_long (hAout, lOutput);
        if (Error) 
        {
            ErrorHandler(Error, "max_write_channel_long");
        }
        // output a ramp from 0 to 4 V
        lOutput += 100000;
        if (lOutput == 4000000)
            lOutput = 0;

/*      // use this to refresh output values that have been written
        Error = max_trigger_channel(hTrigger);
        if (Error) 
        {
            ErrorHandler(Error, "max_trigger_channel");
            break;
        }
*/
        Sleep(200); // approx. 4 Hz bandwidth
    }
    // release library
    max_exit_lib();
    return;
}
