/////////////////////////////////////////////////////////////////////////////
// XAD244TestDlg.cpp
// 
// This is the main file of the sample program.
//
// Windows sample program for the X-AD24-4i. 
// It can be used with any carrier board
//
// (board and CPU initialization is done in XAD244Test.cpp)
// This sample program demponstrates the usage of the MDD for the X-AD24-4i.
// The program allows to open AIN and DIN channels with configurable channel properties.
// Afterwards data can be read from the channels.
// The usage of 
// Copyright SORCUS Computer GmbH 2005
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XAD244Test.h"
#include "XAD244TestDlg.h"
#include "CalibrationValue.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CXAD244TestDlg
/////////////////////////////////////////////////////////////////////////////
CXAD244TestDlg* CXAD244TestDlg::m_pDlg=NULL;

CXAD244TestDlg::CXAD244TestDlg(CWnd* pParent)
:CDialog(CXAD244TestDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CXAD244TestDlg)
    m_nIndexFirst			= 1;
    m_nIndexLast			= 1;
    m_nMode					= 0;
    m_nFilter				= 0x320000;
    m_tHwFormat				= FALSE;
    m_tUncorrected			= FALSE;
    m_tNewDataCallback	= FALSE;
    m_ulSensor				= 0;
    m_nOversampling		= 1;
    m_tDinPosEdge			= FALSE;
    m_tDinNegEdge			= FALSE;
    m_strAin0				= _T("");
    m_strAin1				= _T("");
    m_strAin2				= _T("");
    m_strAin3				= _T("");
    m_strDin0				= _T("");
    m_strDin1				= _T("");
    m_strDin2				= _T("");
    m_strDin3				= _T("");
    m_ulCallbackCount = 0;
    m_tAutoRead = FALSE;
    //}}AFX_DATA_INIT
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
    m_hAin	= 0;
    m_hDin	= 0;
    m_pDlg	= this;
    m_nDinTimer	= 0;
}

/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CXAD244TestDlg)
    DDX_Text(pDX, IDC_INDEX_FIRST_EDIT, m_nIndexFirst);
    DDV_MinMaxUInt(pDX, m_nIndexFirst, 0, 7);
    DDX_Text(pDX, IDC_INDEX_LAST_EDIT, m_nIndexLast);
    DDV_MinMaxUInt(pDX, m_nIndexLast, 0, 7);
    DDX_Radio(pDX, IDC_ACTUAL_MODE_RADIO, m_nMode);
    DDX_Text(pDX, IDC_FILTER_EDIT, m_nFilter);
    DDX_Check(pDX, IDC_HW_FLAG_CHECK, m_tHwFormat);
    DDX_Check(pDX, IDC_UNCORRECTED_CHECK, m_tUncorrected);
    DDX_Check(pDX, IDC_NEW_DATA_CHECK, m_tNewDataCallback);
    DDX_Text(pDX, IDC_SENSOR_EDIT, m_ulSensor);
    DDX_Text(pDX, IDC_OVERSAMPLING_EDIT, m_nOversampling);
    DDV_MinMaxUInt(pDX, m_nOversampling, 1, 64);
    DDX_Check(pDX, IDC_TRIGGER_POS_EDGE_CHECK, m_tDinPosEdge);
    DDX_Check(pDX, IDC_TRIGGER_NEG_EDGE_CHECK, m_tDinNegEdge);
    DDX_Text(pDX, IDC_AIN0_EDIT, m_strAin0);
    DDX_Text(pDX, IDC_AIN1_EDIT, m_strAin1);
    DDX_Text(pDX, IDC_AIN2_EDIT, m_strAin2);
    DDX_Text(pDX, IDC_AIN3_EDIT, m_strAin3);
    DDX_Text(pDX, IDC_DIN0_EDIT, m_strDin0);
    DDX_Text(pDX, IDC_DIN1_EDIT, m_strDin1);
    DDX_Text(pDX, IDC_DIN2_EDIT, m_strDin2);
    DDX_Text(pDX, IDC_DIN3_EDIT, m_strDin3);
    DDX_Text(pDX, IDC_CALLBACK_EDIT, m_ulCallbackCount);
    DDX_Check(pDX, IDC_AUTO_READ_DIN_CHECK, m_tAutoRead);
    //}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CXAD244TestDlg, CDialog)
//{{AFX_MSG_MAP(CXAD244TestDlg)
ON_WM_PAINT()
ON_WM_QUERYDRAGICON()
ON_BN_CLICKED(IDC_OPEN_AIN_BUTTON, OnOpenAinButton)
ON_BN_CLICKED(IDC_READ_AIN_BUTTON, OnReadAinButton)
ON_BN_CLICKED(IDC_CLOSE_AIN_BUTTON, OnCloseAinButton)
ON_BN_CLICKED(IDC_CALIB1_AIN_BUTTON, OnCalib1AinButton)
ON_BN_CLICKED(IDC_CALIB2_AIN_BUTTON, OnCalib2AinButton)
ON_BN_CLICKED(IDC_OPEN_DIN_BUTTON, OnOpenDinButton)
ON_BN_CLICKED(IDC_READ_DIN_BUTTON, OnReadDinButton)
ON_BN_CLICKED(IDC_CLOSE_DIN_BUTTON, OnCloseDinButton)
ON_BN_CLICKED(IDC_LIN_BUTTON, OnLinButton)
ON_BN_CLICKED(IDC_AUTO_READ_DIN_CHECK, OnAutoReadDinCheck)
ON_WM_TIMER()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
BOOL CXAD244TestDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    SetIcon(m_hIcon, TRUE);
    SetIcon(m_hIcon, FALSE);
    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnPaint() 
{
    if (IsIconic())
    {
        CPaintDC dc(this);
        
        SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;
        dc.DrawIcon(x, y, m_hIcon);
    }
    else CDialog::OnPaint();
}

/////////////////////////////////////////////////////////////////////////////
HCURSOR CXAD244TestDlg::OnQueryDragIcon()
{
    return (HCURSOR) m_hIcon;
}

/////////////////////////////////////////////////////////////////////////////
// open ain channel, properties can be selected in the dialog
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnOpenAinButton() 
{
    MAX_CHANNEL_CALLBACK_TYPE	pCbFunc=NULL;
    MAX_ERROR						Error;
    CString							strError;
    
    GetDlgItem(IDC_READ_AIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CALIB1_AIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CALIB2_AIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_LIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CLOSE_AIN_BUTTON)->EnableWindow(FALSE);
    UpdateData(TRUE);
    m_sCpsAin.usVersion			= 1;
    m_sCpsAin.usDevice			= DEVICE_AIN_DIFF;
    m_sCpsAin.usIndexFirst		= m_nIndexFirst;
    m_sCpsAin.usIndexLast		= m_nIndexLast;
    m_sCpsAin.usReadMode		= IO_MODE_LATCH;
    m_sCpsAin.usWriteMode		= 0;
    m_sCpsAin.ulFlags			= 0;
    if (m_tHwFormat)
        m_sCpsAin.ulFlags		|= _CP_HW_FORMAT;
    if (m_tUncorrected)
        m_sCpsAin.ulFlags		|= _CP_UNCORRECTED;
    switch (m_nMode)
    {
    case 0:
        m_sCpsAin.ulMode		= XAD244I_MODE_ACTUAL_VALUE;
        break;
    case 1:
        m_sCpsAin.ulMode		= XAD244I_MODE_NEW_VALUE;
        break;
    case 2:
        m_sCpsAin.ulMode		= XAD244I_MODE_TRIGGER;
        break;
    case 3:
        m_sCpsAin.ulMode		= XAD244I_MODE_CONTINUOUS;
        break;
    }
    m_sCpsAin.ulFilter			= m_nFilter;
    m_sCpsAin.ulSensor			= m_ulSensor;
    m_sCpsAin.ulTrigger			= 0;
    if (m_tDinPosEdge)
        m_sCpsAin.ulTrigger		|= _XAD244I_DIN_POS_EDGE;
    if (m_tDinNegEdge)
        m_sCpsAin.ulTrigger		|= _XAD244I_DIN_NEG_EDGE;
    m_sCpsAin.ulCallbackEvents	= 0;
    if (m_tNewDataCallback)
    {
        m_sCpsAin.ulCallbackEvents  |= _XAD244I_EVENT_NEW_DATA;
        pCbFunc = CbNewData;
    }
    m_sCpsAin.usOversampling	= m_nOversampling;
    // open channel
    if ((Error=max_open_channel(m_hMdd, sizeof(m_sCpsAin), &m_sCpsAin, pCbFunc, 0, &m_hAin))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_open_channel: 0x%X"), Error);
        AfxMessageBox(strError);
        m_hAin = 0;
    }
    else
    {
        GetDlgItem(IDC_READ_AIN_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_CALIB1_AIN_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_CALIB2_AIN_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_LIN_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_CLOSE_AIN_BUTTON)->EnableWindow(TRUE);
        // the format of the return values depends on the sensor type, i.e. the module variant
        // the module variant was detected on startup with max_get_module_info
        // for the later calculation of the values to be read a divider is set here:
        switch (m_usSubType)
        {
        case 0:	    // X-AD24-4i/S
            m_dDivider	= 10000000;		
            break;
        case 1:	    // X-AD24-4i/T
            if (m_sCpsAin.usIndexFirst < 4 && 
                m_sCpsAin.ulSensor == XAD244I_SENSOR_UNDEFINED)
            {
                m_dDivider	= 1000000;		
            }
            else    // channel for cold junction compensation or 
            {       // self defined linearisation table
                m_dDivider	= 10000;		
            }
            break;
        case 2:	    // X-AD24-4i/P
            m_dDivider	= 10000;			
            break;
        case 3:	    // X-AD24-4i/I
            m_dDivider	= 1000000;		
            break;
        case 4:	    // X-AD24-4i/C
            m_dDivider	= 1000000;		
            break;
        case 5:	    // X-AD24-4i/V
            m_dDivider	= 1000000;		
            break;
        case 6:	    // X-AD24-4i/F
            m_dDivider	= 10000000;		
            break;  
        default:    // custom specific
            m_dDivider	= 100000;		
            break;
        }
    }
}

/////////////////////////////////////////////////////////////////////////////
// read ain and show result
// This function must distinguish if it's a single or multiple channel 
// Displaying the value is different for hardware format and normalized format.
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnReadAinButton() 
{
    USHORT		usIndexFirst, usIndexLast;
    ULONG		ulSize;
    long		lData[4];
    MAX_ERROR	Error;
    
    usIndexFirst = m_sCpsAin.usIndexFirst;
    usIndexLast = m_sCpsAin.usIndexLast;
    if (usIndexFirst>=4)    // it's a channel for cold jection compensation
    {
        usIndexFirst-= 4;
        usIndexLast-= 4;
    }
    if (usIndexFirst==usIndexLast)
    {   // single input channel
        if ((Error=max_read_channel_long(m_hAin, &lData[usIndexFirst]))!=ERR_OK)
            max_clear_error();
    }
    else
    {   // multiple input channel
        ulSize = 4*(usIndexLast-usIndexFirst+1);
        if ((Error=max_read_channel_block(m_hAin, &ulSize, &lData[usIndexFirst]))!=ERR_OK)
            max_clear_error();
    }
    if (usIndexFirst==0)
    {
        if (Error!=ERR_OK)
            m_strAin0.Format(_T("Error: 0x%X"), Error);
        else if (m_sCpsAin.ulFlags & _CP_HW_FORMAT)
            m_strAin0.Format(_T("0x%X"), lData[0]);
        else
            m_strAin0.Format(_T("%f (0x%X)"), (double)lData[0]/m_dDivider, lData[0]);
    }
    if (usIndexFirst<=1 && usIndexLast>=1)
    {
        if (Error!=ERR_OK)
            m_strAin1.Format(_T("Error: 0x%X"), Error);
        else if (m_sCpsAin.ulFlags & _CP_HW_FORMAT)
            m_strAin1.Format(_T("0x%X"), lData[1]);
        else
            m_strAin1.Format(_T("%f (0x%X)"), (double)lData[1]/m_dDivider, lData[1]);
    }
    if (usIndexFirst<=2 && usIndexLast>=2)
    {
        if (Error!=ERR_OK)
            m_strAin2.Format(_T("Error: 0x%X"), Error);
        else if (m_sCpsAin.ulFlags & _CP_HW_FORMAT)
            m_strAin2.Format(_T("0x%X"), lData[2]);
        else
            m_strAin2.Format(_T("%f (0x%X)"), (double)lData[2]/m_dDivider, lData[2]);
    }
    if (usIndexFirst<=3 && usIndexLast>=3)
    {
        if (Error!=ERR_OK)
            m_strAin3.Format(_T("Error: 0x%X"), Error);
        else if (m_sCpsAin.ulFlags & _CP_HW_FORMAT)
            m_strAin3.Format(_T("0x%X"), lData[3]);
        else
            m_strAin3.Format(_T("%f (0x%X)"), (double)lData[3]/m_dDivider, lData[3]);
    }
    UpdateData(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// close ain
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnCloseAinButton() 
{
    MAX_ERROR		Error;
    CString			strError;
    
    if ((Error=max_close_channel(&m_hAin))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_close_channel: 0x%X"), Error);
        AfxMessageBox(strError);
    }
    m_hAin = 0;
    GetDlgItem(IDC_READ_AIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CALIB1_AIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CALIB2_AIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_LIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CLOSE_AIN_BUTTON)->EnableWindow(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// calibrate ain (can be done with SNW32)
// apply reference 1 to the channel and call the callibration service
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnCalib1AinButton() 
{
    CCalibrationValue		CalibDlg;
    LONG						lRef;
    MAX_ERROR				Error;
    CString					strError;
    
    if (CalibDlg.DoModal()!=IDOK)
        return;
    lRef = CalibDlg.m_lValue;
    if ((Error=max_channel_control(m_hAin, CTRL_CALIB_REFVAL1, 4, &lRef))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_channel_control: 0x%X"), Error);
        AfxMessageBox(strError);
    }
}
/////////////////////////////////////////////////////////////////////////////
// calibrate ain
// apply reference 2 to the channel and call the callibration service
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnCalib2AinButton() 
{
    CCalibrationValue		CalibDlg;
    LONG						lRef;
    MAX_ERROR				Error;
    CString					strError;
    
    if (CalibDlg.DoModal()!=IDOK)
        return;
    lRef = CalibDlg.m_lValue;
    if ((Error=max_channel_control(m_hAin, CTRL_CALIB_REFVAL2, 4, &lRef))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_channel_control: 0x%X"), Error);
        AfxMessageBox(strError);
    }
}
/////////////////////////////////////////////////////////////////////////////
// linearize ain
// show how to retrieve the channel's actual linearisation table and 
// how to change the linearisation table 
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnLinButton() 
{
    XAD244I_LIN_TABLE		rcLinTable;
    USHORT					usSize;
    MAX_ERROR				Error;
    CString					strError;
    
    memset(&rcLinTable, 0, sizeof(XAD244I_LIN_TABLE));
    usSize = sizeof(XAD244I_LIN_TABLE);
    if ((Error=max_channel_info(m_hAin, INFO_LINEARIZATION_TABLE, &usSize, &rcLinTable))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_channel_info: 0x%X"), Error);
        AfxMessageBox(strError);
    }
    memset(&rcLinTable, 0, sizeof(XAD244I_LIN_TABLE));
    rcLinTable.usVersion= 1;
    rcLinTable.usSize	= 40;
    rcLinTable.ulSensor	= 1;
    rcLinTable.lIn[0]	= 1849000;
    rcLinTable.lOut[0]	= -20000000;
    rcLinTable.lIn[1]	= 2920000;
    rcLinTable.lOut[1]	= -17500000;
    rcLinTable.lIn[2]	= 3971000;
    rcLinTable.lOut[2]	= -15000000;
    rcLinTable.lIn[3]	= 5006000;
    rcLinTable.lOut[3]	= -12500000;
    rcLinTable.lIn[4]	= 6025000;
    rcLinTable.lOut[4]	= -10000000;
    rcLinTable.lIn[5]	= 6430000;
    rcLinTable.lOut[5]	= -9000000;
    rcLinTable.lIn[6]	= 6833000;
    rcLinTable.lOut[6]	= -8000000;
    rcLinTable.lIn[7]	= 7233000;
    rcLinTable.lOut[7]	= -7000000;
    rcLinTable.lIn[8]	= 7633000;
    rcLinTable.lOut[8]	= -6000000;
    rcLinTable.lIn[9]	= 8031000;
    rcLinTable.lOut[9]	= -5000000;
    rcLinTable.lIn[10]	= 8427000;
    rcLinTable.lOut[10]	= -4000000;
    rcLinTable.lIn[11]	= 8822000;
    rcLinTable.lOut[11]	= -3000000;
    rcLinTable.lIn[12]	= 9216000;
    rcLinTable.lOut[12]	= -2000000;
    rcLinTable.lIn[13]	= 9609000;
    rcLinTable.lOut[13]	= -1000000;
    rcLinTable.lIn[14]	= 10000000;
    rcLinTable.lOut[14]	= 0;
    rcLinTable.lIn[15]	= 10195000;
    rcLinTable.lOut[15]	= 500000;
    rcLinTable.lIn[16]	= 10390000;
    rcLinTable.lOut[16]	= 1000000;
    rcLinTable.lIn[17]	= 10585000;
    rcLinTable.lOut[17]	= 1500000;
    rcLinTable.lIn[18]	= 10779000;
    rcLinTable.lOut[18]	= 2000000;
    rcLinTable.lIn[19]	= 10973000;
    rcLinTable.lOut[19]	= 2500000;
    rcLinTable.lIn[20]	= 11167000;
    rcLinTable.lOut[20]	= 3000000;
    rcLinTable.lIn[21]	= 11554000;
    rcLinTable.lOut[21]	= 4000000;
    rcLinTable.lIn[22]	= 11940000;
    rcLinTable.lOut[22]	= 5000000;
    rcLinTable.lIn[23]	= 12324000;
    rcLinTable.lOut[23]	= 6000000;
    rcLinTable.lIn[24]	= 12707000;
    rcLinTable.lOut[24]	= 7000000;
    rcLinTable.lIn[25]	= 13089000;
    rcLinTable.lOut[25]	= 8000000;
    rcLinTable.lIn[26]	= 13470000;
    rcLinTable.lOut[26]	= 9000000;
    rcLinTable.lIn[27]	= 13850000;
    rcLinTable.lOut[27]	= 10000000;
    rcLinTable.lIn[28]	= 14229000;
    rcLinTable.lOut[28]	= 11000000;
    rcLinTable.lIn[29]	= 14606000;
    rcLinTable.lOut[29]	= 12000000;
    rcLinTable.lIn[30]	= 14982000;
    rcLinTable.lOut[30]	= 13000000;
    rcLinTable.lIn[31]	= 15358000;
    rcLinTable.lOut[31]	= 14000000;
    rcLinTable.lIn[32]	= 15731000;
    rcLinTable.lOut[32]	= 15000000;
    rcLinTable.lIn[33]	= 16104000;
    rcLinTable.lOut[33]	= 16000000;
    rcLinTable.lIn[34]	= 16476000;
    rcLinTable.lOut[34]	= 17000000;
    rcLinTable.lIn[35]	= 16846000;
    rcLinTable.lOut[35]	= 18000000;
    rcLinTable.lIn[36]	= 17216000;
    rcLinTable.lOut[36]	= 19000000;
    rcLinTable.lIn[37]	= 17584000;
    rcLinTable.lOut[37]	= 20000000;
    rcLinTable.lIn[38]	= 18499000;
    rcLinTable.lOut[38]	= 22500000;
    rcLinTable.lIn[39]	= 19407000;
    rcLinTable.lOut[39]	= 25000000;
    rcLinTable.lIn[40]	= 20308000;
    rcLinTable.lOut[40]	= 27500000;
    rcLinTable.lIn[41]	= 21202000;
    rcLinTable.lOut[41]	= 30000000;
    rcLinTable.lIn[42]	= 22088000;
    rcLinTable.lOut[42]	= 32500000;
    rcLinTable.lIn[43]	= 22967000;
    rcLinTable.lOut[43]	= 35000000;
    rcLinTable.lIn[44]	= 23839000;
    rcLinTable.lOut[44]	= 37500000;
    rcLinTable.lIn[45]	= 24704000;
    rcLinTable.lOut[45]	= 40000000;
    rcLinTable.lIn[46]	= 25561000;
    rcLinTable.lOut[46]	= 42500000;
    rcLinTable.lIn[47]	= 26411000;
    rcLinTable.lOut[47]	= 45000000;
    rcLinTable.lIn[48]	= 27254000;
    rcLinTable.lOut[48]	= 47500000;
    rcLinTable.lIn[49]	= 28090000;
    rcLinTable.lOut[49]	= 50000000;
    rcLinTable.lIn[50]	= 28918000;
    rcLinTable.lOut[50]	= 52500000;
    rcLinTable.lIn[51]	= 29739000;
    rcLinTable.lOut[51]	= 55000000;
    rcLinTable.lIn[52]	= 30553000;
    rcLinTable.lOut[52]	= 57500000;
    rcLinTable.lIn[53]	= 31359000;
    rcLinTable.lOut[53]	= 60000000;
    rcLinTable.lIn[54]	= 32159000;
    rcLinTable.lOut[54]	= 62500000;
    rcLinTable.lIn[55]	= 32951000;
    rcLinTable.lOut[55]	= 65000000;
    rcLinTable.lIn[56]	= 33736000;
    rcLinTable.lOut[56]	= 67500000;
    rcLinTable.lIn[57]	= 34513000;
    rcLinTable.lOut[57]	= 70000000;
    rcLinTable.lIn[58]	= 35283000;
    rcLinTable.lOut[58]	= 72500000;
    rcLinTable.lIn[59]	= 36047000;
    rcLinTable.lOut[59]	= 75000000;
    rcLinTable.lIn[60]	= 36802000;
    rcLinTable.lOut[60]	= 77500000;
    rcLinTable.lIn[61]	= 37551000;
    rcLinTable.lOut[61]	= 80000000;
    rcLinTable.lIn[62]	= 38292000;
    rcLinTable.lOut[62]	= 82500000;
    rcLinTable.lIn[63]	= 39026000;
    rcLinTable.lOut[63]	= 85000000;
    if ((Error=max_channel_control(m_hAin, CTRL_LINEARIZATION_TABLE, sizeof(XAD244I_LIN_TABLE), &rcLinTable))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_channel_control: 0x%X"), Error);
        AfxMessageBox(strError);
    }
}

/////////////////////////////////////////////////////////////////////////////
// callback function: called when the edge given in ulTrigger occurs
/////////////////////////////////////////////////////////////////////////////
MAX_CALLBACK CXAD244TestDlg::CbNewData(MAXCHLHND hChannel, DWORD dwParam, DWORD dwSize, void* pData)
{
    m_pDlg->m_ulCallbackCount++;
}

/////////////////////////////////////////////////////////////////////////////
// open din
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnOpenDinButton() 
{
    MAX_ERROR		Error;
    CString			strError;
    
    GetDlgItem(IDC_READ_DIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_AUTO_READ_DIN_CHECK)->EnableWindow(FALSE);
    GetDlgItem(IDC_CLOSE_DIN_BUTTON)->EnableWindow(FALSE);
    UpdateData(TRUE);
    m_sCpsDin.usVersion				= 1;
    m_sCpsDin.usDevice				= DEVICE_DIN;
    m_sCpsDin.usIndexFirst			= m_nIndexFirst;
    m_sCpsDin.usIndexLast			= m_nIndexLast;
    m_sCpsDin.usReadMode			= IO_MODE_LATCH;
    m_sCpsDin.usWriteMode			= 0;
    m_sCpsDin.ulFlags				= 0;
    m_sCpsDin.ulMode				= 0;
    m_sCpsDin.ulFilter				= 0;
    m_sCpsDin.ulSensor				= 0;
    m_sCpsDin.ulTrigger				= 0;
    m_sCpsDin.ulCallbackEvents		= 0;
    m_sCpsDin.usOversampling		= 0;
    if ((Error=max_open_channel(m_hMdd, sizeof(m_sCpsDin), &m_sCpsDin, NULL, 0, &m_hDin))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_open_channel: 0x%X"), Error);
        AfxMessageBox(strError);
        m_hDin = 0;
    }
    else
    {
        GetDlgItem(IDC_READ_DIN_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_AUTO_READ_DIN_CHECK)->EnableWindow(TRUE);
        GetDlgItem(IDC_CLOSE_DIN_BUTTON)->EnableWindow(TRUE);
    }
}

/////////////////////////////////////////////////////////////////////////////
// read din
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnReadDinButton() 
{
    UCHAR			ucData, ucMask=0x01;
    MAX_ERROR	Error;
    CString		strError;
    
    if (m_hDin==NULL)
        return;
    m_strDin0 = _T("");
    m_strDin1 = _T("");
    m_strDin2 = _T("");
    m_strDin3 = _T("");
    if ((Error=max_read_channel_uchar(m_hDin, &ucData))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_read_channel_uchar: 0x%X"), Error);
        AfxMessageBox(strError);
    }
    else
    {
        if (m_sCpsDin.usIndexFirst==0)
        {
            if ((ucData & ucMask)!=0)
                m_strDin0 = _T("1");
            else
                m_strDin0 = _T("0");
            ucMask<<= 1;
        }
        if (m_sCpsDin.usIndexFirst<=1 && m_sCpsDin.usIndexLast>=1)
        {
            if ((ucData & ucMask)!=0)
                m_strDin1 = _T("1");
            else
                m_strDin1 = _T("0");
            ucMask<<= 1;
        }
        if (m_sCpsDin.usIndexFirst<=2 && m_sCpsDin.usIndexLast>=2)
        {
            if ((ucData & ucMask)!=0)
                m_strDin2 = _T("1");
            else
                m_strDin2 = _T("0");
            ucMask<<= 1;
        }
        if (m_sCpsDin.usIndexFirst<=3 && m_sCpsDin.usIndexLast>=3)
        {
            if ((ucData & ucMask)!=0)
                m_strDin3 = _T("1");
            else
                m_strDin3 = _T("0");
        }
    }
    UpdateData(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// auto read din
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnAutoReadDinCheck() 
{
    m_tAutoRead = !m_tAutoRead;
    if (m_tAutoRead)
        m_nDinTimer = SetTimer(1, 1000, NULL);
    else
        KillTimer(m_nDinTimer);
    UpdateData(TRUE);	
}

/////////////////////////////////////////////////////////////////////////////
// close din
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnCloseDinButton() 
{
    MAX_ERROR		Error;
    CString			strError;
    
    if ((Error=max_close_channel(&m_hDin))!=ERR_OK)
    {
        max_clear_error();
        strError.Format(_T("max_close_channel: 0x%X"), Error);
        AfxMessageBox(strError);
    }
    m_hDin = 0;
    GetDlgItem(IDC_READ_DIN_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_AUTO_READ_DIN_CHECK)->EnableWindow(FALSE);
    GetDlgItem(IDC_CLOSE_DIN_BUTTON)->EnableWindow(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// timer
/////////////////////////////////////////////////////////////////////////////
void CXAD244TestDlg::OnTimer(UINT nIDEvent) 
{
    if (nIDEvent==m_nDinTimer)
    {
        OnReadDinButton();
        return;
    }
    CDialog::OnTimer(nIDEvent);
}


