/****************************************************************************
*   FREQUENCY.CPP   
*
*   PURPOSE
*       The program show handling of X-C16-3 module-device-driver. It shows
*		how to use the frequency counter. 
*   COPYRIGHT
*       (C)2003 SORCUS Computer GmbH
*   HISTORY
*       - 2003-10-24 (JS) First Version
****************************************************************************/

// timer.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "windows.h"
#include "iostream.h"
#include "conio.h"

// include SORCUS header files
#include "max_lib.h"
#include "x_mdd.h"

// add SORCUS maxw32.lib
#pragma comment( lib, "c:\\sorcus\\maxlib\\pc\\win32\\msvc\\maxw32" )

// some global variables
USHORT g_usBoardNo, g_usCpuSlot, g_usCpuLayer, g_usXc163Slot, g_usXc163Layer;
MAXMODHND g_hCpu;
MAXMDDHND g_hXc163Mdd;
MAXCHLHND g_hFrequency;




//*****************************************************************************
// HandleError
/**
* 
* @param    pcText
* @param    Error
* @return   -
*/
//*****************************************************************************
void HandleError(CHAR* pcText, MAX_ERROR Error)
{
    CHAR acMessage[200];
    USHORT usSize = 200;

    max_clear_error();
    max_get_error_message(Error, &usSize, acMessage);
    cout << "-------------------------------------------------------------------" << endl;
    cout << "Error at        : " << pcText << endl;
    cout << "Error-Code      : " << hex << Error << dec << "hex" << endl;
    cout << "Error-Message   : " << acMessage << endl;
    cout << "-------------------------------------------------------------------" << endl;
    return;
}



//*****************************************************************************
// main
/**
*
* @param    argc
* @param    argv
* @return   int
*/
//*****************************************************************************
int main(int argc, char* argv[])
{
    USHORT usData, i;
    ULONG ulSize;
    MAX_ERROR Error;
    MAX_OS_INFO rcOs;
    CPS_XC163 rcFrequency;

    // init global variables
    g_usBoardNo = 0;        // number of board
    g_usCpuSlot = 1;        // cpu slot
    g_usCpuLayer = 0;       // cpu layer
    g_usXc163Slot = 6;      // X-C16-3 slot
    g_usXc163Layer = 0;     // X-C16-3 layer

    // init library
    Error = max_init_lib(0);
    
    // reset board
    Error = max_reset_board(g_usBoardNo, 20);
    if (Error != ERR_OK)
    {
        HandleError("max_reset_board", Error);
        _getch();
        return 1;
    }
    
    // connect cpu
    Error = max_connect_cpu (g_usBoardNo, g_usCpuSlot, g_usCpuLayer, &rcOs, &g_hCpu);
    if (Error != ERR_OK)
    {
        HandleError("max_connect_cpu", Error);
        _getch();
        return 1;
    }

    // load osx
    Error = max_load_osx (g_hCpu, "ROM");
    if (Error != ERR_OK)
    {
        HandleError("max_load_osx", Error);
        _getch();
        return 1;
    }

    // load mdd
    Error = max_load_mdd (g_hCpu, g_usXc163Slot, g_usXc163Layer, 0, MDD_X_C16_3, NULL, &g_hXc163Mdd);
    if (Error != ERR_OK)
    {
        HandleError("max_load_mdd", Error);
        _getch();
        return 1;
    }

    // open frequency channel
    rcFrequency.usDevice = DEVICE_FREQUENCY_COUNTER;
    rcFrequency.usIndexFirst = 0;
    rcFrequency.usIndexLast = 0;
    rcFrequency.usFlags = _CP_EXCLUSIVE;
    rcFrequency.usReadMode = IO_MODE_LATCH;
    rcFrequency.usWriteMode = IO_MODE_DIRECT;
    rcFrequency.usMode = _XC163_AUTORUN | _XC163_OVERRIDE | _XC163_CONTINUOUS;
    rcFrequency.usReferenceTime = XC163_1MS;
    rcFrequency.usCounterInput = _XC163_DIN_0_1;

    Error = max_open_channel (g_hXc163Mdd, sizeof (rcFrequency), &rcFrequency, NULL, 0, &g_hFrequency);
    if (Error != ERR_OK)
    {
        HandleError("max_open_channel", Error);
        _getch();
        return 1;
    }

    for (i=0; i<10; i++)
    {
        // read channel
        ulSize = 2;
        Error = max_read_channel_block(g_hFrequency, &ulSize, (void*)&usData);
        if (Error != ERR_OK)
        {
            HandleError("max_write_channel", Error);
            _getch();
            return 1;
        }

        cout << "Data: " << usData << endl;

        Sleep(5000);
    }


    cout << "End of example" << endl;
    _getch();

    return 0;
}
