/****************************************************************************
*	CALLBACK.CPP	
*
*	PURPOSE
*		Sample program
* 		The program show handling of X-DIO-40/i module-device-driver.
*       Be sure that you will use the multithreaded version of the runtime
*       library.
*	COPYRIGHT
*		(C) 2003 SORCUS Computer GmbH
*	HISTORY
*		- 2003-10-21 (JS) First Version
****************************************************************************/

// callback.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "windows.h"
#include "iostream.h"
#include "conio.h"

// include SORCUS header files
#include "max_lib.h"
#include "x_mdd.h"

// add SORCUS maxw32.lib
#pragma comment( lib, "c:\\sorcus\\maxlib\\pc\\win32\\msvc\\maxw32" )

// some global variables
USHORT g_usBoardNo, g_usCpuSlot, g_usCpuLayer, g_usXdioSlot, g_usXdioLayer;
ULONG g_ulCounter;
MAXMODHND g_hCpu;
MAXMDDHND g_hXdioMdd;
MAXCHLHND g_hLed, g_hDin;




//*****************************************************************************
// HandleError
/**
* 
* @param    pcText
* @param    Error
* @return   -
*/
//*****************************************************************************
void HandleError(CHAR* pcText, MAX_ERROR Error)
{
    CHAR acMessage[200];
    USHORT usSize = 200;

    max_clear_error();
    max_get_error_message(Error, &usSize, acMessage);
    cout << "-------------------------------------------------------------------" << endl;
    cout << "Error at        : " << pcText << endl;
    cout << "Error-Code      : " << hex << Error << dec << "hex" << endl;
    cout << "Error-Message   : " << acMessage << endl;
    cout << "-------------------------------------------------------------------" << endl;
    return;
}



//*****************************************************************************
// MyCallbackFunction
/**
*
* @param	handle
* @param	param
* @param	size
* @param	pData
* @return	MAX_CALLBACK
*/
//*****************************************************************************
MAX_CALLBACK MyCallbackFunction(MAXCHLHND handle, ULONG param, ULONG size, void* pData)
{
	MAX_ERROR Error;
    UCHAR ucLed;

    g_ulCounter++;
    cout << "Callback-Funktion has been called " << g_ulCounter << " times" << endl;

	// toggle LED
	Error = max_read_channel_uchar(g_hLed, &ucLed);
	if (ucLed != 0)
	{
		Error = max_write_channel_uchar(g_hLed, 0);
	}
	else
	{
		Error = max_write_channel_uchar(g_hLed, 1);
	}

    if (Error != ERR_OK)
    {
        HandleError("max_reset_board", Error);
    }
}



//*****************************************************************************
// main
/**
*
* @param    argc
* @param    argv
* @return   int
*/
//*****************************************************************************
int main(int argc, char* argv[])
{
    USHORT i;
    MAX_ERROR Error;
    MAX_OS_INFO rcOs;
    CPS_XDIO40_A   rcDin, rcLed;

    // init global variables
    g_usBoardNo = 0;
    g_usCpuSlot = 1;
    g_usCpuLayer = 0;
    g_usXdioSlot = 6;
    g_usXdioLayer = 0;
    g_ulCounter = 0;

    // init library
    Error = max_init_lib(0);
    
    // reset board
    Error = max_reset_board(g_usBoardNo, 20);
    if (Error != ERR_OK)
    {
        HandleError("max_reset_board", Error);
        _getch();
        return 1;
    }
    
    // connect cpu
    Error = max_connect_cpu (g_usBoardNo, g_usCpuSlot, g_usCpuLayer, &rcOs, &g_hCpu);
    if (Error != ERR_OK)
    {
        HandleError("max_connect_cpu", Error);
        _getch();
        return 1;
    }

    // load osx
    Error = max_load_osx (g_hCpu, "ROM");
    if (Error != ERR_OK)
    {
        HandleError("max_load_osx", Error);
        _getch();
        return 1;
    }

    // load mdd
    Error = max_load_mdd (g_hCpu, g_usXdioSlot, g_usXdioLayer, 0, MDD_X_DIO_40, NULL, &g_hXdioMdd);
    if (Error != ERR_OK)
    {
        HandleError("max_load_mdd", Error);
        _getch();
        return 1;
    }

    // open LED channel
    rcLed.usDevice      = DEVICE_LED;
    rcLed.usIndexFirst  = 0;
    rcLed.usIndexLast   = 0;
    rcLed.usReadMode    = IO_MODE_DIRECT;
    rcLed.usWriteMode   = IO_MODE_DIRECT;
    rcLed.usFlags       = 0;
    rcLed.usMode        = 0;
    Error = max_open_channel(g_hXdioMdd, sizeof (rcLed), &rcLed, NULL, 0, &g_hLed);

    // open DIN channel 0 ... 3
    rcDin.usDevice      = DEVICE_DIN;
    rcDin.usIndexFirst  = 0;
    rcDin.usIndexLast   = 3;
    rcDin.usReadMode    = IO_MODE_DIRECT;
    rcDin.usWriteMode   = 0;
    rcDin.usFlags       = _CP_EXCLUSIVE;
    rcDin.usMode        = 0;
    for (i=0; i<3; i++)
    {
        rcDin.rcCallbackEvents.ausPosEdge[i] = 0;
        rcDin.rcCallbackEvents.ausNegEdge[i] = 0;
    }
    rcDin.rcCallbackEvents.ausPosEdge[0] = 0x2;

    Error = max_open_channel(g_hXdioMdd, sizeof (rcDin), &rcDin, MyCallbackFunction, 0, &g_hDin);

    // check error
    if (Error != ERR_OK)
    {
        HandleError("max_open_channel", Error);
        _getch();
        return 1;
    }

    // for loop
    cout << "Running \"for loop\" for 30 sec ..." << endl;
    for (i=0; i<300; i++)
    {
        Sleep(100);
    }

    cout << "End of example" << endl;

    return 0;
}
