/****************************************************************************
*	DINDOUT.CPP	
*
*	PURPOSE
*		Sample program
* 		The program show handling of X-DIO-40/i, X-DIO-40 and X-DIO-32 
*		module-device-driver.
*	COPYRIGHT
*		(C)2003 SORCUS Computer GmbH
*	HISTORY
*		- 2003-10-21 (JS) First Version
****************************************************************************/

// dindout.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "windows.h"
#include "iostream.h"
#include "conio.h"

// include SORCUS header files
#include "max_lib.h"
#include "x_mdd.h"

// add SORCUS maxw32.lib
#pragma comment( lib, "c:\\sorcus\\maxlib\\pc\\win32\\msvc\\maxw32" )

// some global variables
USHORT g_usBoardNo, g_usCpuSlot, g_usCpuLayer, g_usXdioSlot, g_usXdioLayer;
MAXMODHND g_hCpu;
MAXMDDHND g_hXdioMdd;
MAXCHLHND g_hLed, g_hDin, g_hDout, g_hDio;




//*****************************************************************************
// HandleError
/**
* 
* @param    pcText
* @param    Error
* @return   -
*/
//*****************************************************************************
void HandleError(CHAR* pcText, MAX_ERROR Error)
{
    CHAR acMessage[200];
    USHORT usSize = 200;

    max_clear_error();
    max_get_error_message(Error, &usSize, acMessage);
    cout << "-------------------------------------------------------------------" << endl;
    cout << "Error at        : " << pcText << endl;
    cout << "Error-Code      : " << hex << Error << dec << "hex" << endl;
    cout << "Error-Message   : " << acMessage << endl;
    cout << "-------------------------------------------------------------------" << endl;
    return;
}



//*****************************************************************************
// main
/**
*
* @param    argc
* @param    argv
* @return   int
*/
//*****************************************************************************
int main(int argc, char* argv[])
{
    UCHAR ucLed, ucDinData, ucOutData;
    USHORT i;
    MAX_ERROR Error;
    MAX_OS_INFO rcOs;
    CPS_XDIO40_A   rcDin, rcDout, rcDio, rcLed;

    // init global variables
    g_usBoardNo = 0;
    g_usCpuSlot = 1;
    g_usCpuLayer = 0;
    g_usXdioSlot = 6;
    g_usXdioLayer = 0;


    // init library
    Error = max_init_lib(0);
    
    // reset board
    Error = max_reset_board(g_usBoardNo, 20);
    if (Error != ERR_OK)
    {
        HandleError("max_reset_board", Error);
        _getch();
        return 1;
    }
    
    // connect cpu
    Error = max_connect_cpu (g_usBoardNo, g_usCpuSlot, g_usCpuLayer, &rcOs, &g_hCpu);
    if (Error != ERR_OK)
    {
        HandleError("max_connect_cpu", Error);
        _getch();
        return 1;
    }

    // load osx
    Error = max_load_osx (g_hCpu, "ROM");
    if (Error != ERR_OK)
    {
        HandleError("max_load_osx", Error);
        _getch();
        return 1;
    }

    // load mdd
    Error = max_load_mdd (g_hCpu, g_usXdioSlot, g_usXdioLayer, 0, MDD_X_DIO_40, NULL, &g_hXdioMdd);
    if (Error != ERR_OK)
    {
        HandleError("max_load_mdd", Error);
        _getch();
        return 1;
    }

    // open LED channel
    rcLed.usDevice      = DEVICE_LED;
    rcLed.usIndexFirst  = 0;
    rcLed.usIndexLast   = 0;
    rcLed.usReadMode    = IO_MODE_DIRECT;
    rcLed.usWriteMode   = IO_MODE_DIRECT;
    rcLed.usFlags       = 0;
    rcLed.usMode        = 0;
    Error = max_open_channel(g_hXdioMdd, sizeof (rcLed), &rcLed, NULL, 0, &g_hLed);

    // open DIN channel 0 ... 3
    rcDin.usDevice      = DEVICE_DIN;
    rcDin.usIndexFirst  = 0;
    rcDin.usIndexLast   = 3;
    rcDin.usReadMode    = IO_MODE_DIRECT;
    rcDin.usWriteMode   = 0;
    rcDin.usFlags       = _CP_EXCLUSIVE;
    rcDin.usMode        = 0;
    for (i=0; i<3; i++)
    {
        rcDin.rcCallbackEvents.ausPosEdge[i] = 0;
        rcDin.rcCallbackEvents.ausNegEdge[i] = 0;
    }

    Error = max_open_channel(g_hXdioMdd, sizeof (rcDin), &rcDin, NULL, 0, &g_hDin);

    // open DOUT channel  4 ... 7
    rcDout.usDevice      = DEVICE_DOUT;
    rcDout.usIndexFirst  = 4;
    rcDout.usIndexLast   = 7;
    rcDout.usReadMode    = IO_MODE_DIRECT;
    rcDout.usWriteMode   = IO_MODE_DIRECT;
    rcDout.usFlags       = _CP_EXCLUSIVE;
    rcDout.usMode        = 0;
    for (i=0; i<3; i++)
    {
        rcDout.rcCallbackEvents.ausPosEdge[i] = 0;
        rcDout.rcCallbackEvents.ausNegEdge[i] = 0;
    }
    Error = max_open_channel(g_hXdioMdd, sizeof (rcDout), &rcDout, NULL, 0, &g_hDout);

    // open DIO channel 8 ... 11
    rcDio.usDevice      = DEVICE_DIN;
    rcDio.usIndexFirst  = 8;
    rcDio.usIndexLast   = 11;
    rcDio.usReadMode    = IO_MODE_DIRECT;
    rcDio.usWriteMode   = IO_MODE_DIRECT;
    rcDio.usFlags       = _CP_EXCLUSIVE;
    rcDio.usMode        = 0;
    for (i=0; i<3; i++)
    {
        rcDio.rcCallbackEvents.ausPosEdge[i] = 0;
        rcDio.rcCallbackEvents.ausNegEdge[i] = 0;
    }
    Error = max_open_channel(g_hXdioMdd, sizeof (rcDio), &rcDio, NULL, 0, &g_hDio);

    // check error
    if (Error != ERR_OK)
    {
        HandleError("max_open_channel", Error);
        _getch();
        return 1;
    }

    cout << "Running \"for loop\" for 10 sec ..." << endl;
    for (i=0; i<100; i++)
    {
        // read LED channel
        Error = max_read_channel_uchar(g_hLed, &ucLed);

        // read DIN-0 ... 3
        Error = max_read_channel_uchar(g_hDin, &ucDinData);

        // set DOUT-4 ... 7
        Error = max_write_channel_uchar(g_hDout, ucDinData);

        // set LED
        ucOutData = ucDinData & 0x1;
        Error = max_write_channel_uchar(g_hLed, ucOutData);

        // toggle DIO-8 ... 11 channel
        if ((ucOutData & 0x1) != 0)
        {
            // DIO-8 ... 11 are outputs
            Error = max_channel_control(g_hDio, CMD_DIR_OUTPUT, 0, NULL);
        }
        else
        {
            // DIO-8 ... 11 are inputs
            Error = max_channel_control(g_hDio, CMD_DIR_INPUT, 0, NULL);
        }

        // check error
        if (Error != ERR_OK)
        {
            HandleError("max_open_channel", Error);
            _getch();
            return 1;
        }

        // wait 100 ms
        Sleep(100);
    }


    cout << "End of example" << endl;
    _getch();

    return 0;
}
