/****************************************************************************
*   TIMER.CPP   
*
*   PURPOSE
*       The program show handling of X-DIO-40/i module-device-driver. It shows
*		how to use the timer. We use the square wave mode to generate a 
*		square-wave signal at DOUT-18. 
*   COPYRIGHT
*       (C)2003 SORCUS Computer GmbH
*   HISTORY
*       - 2003-10-21 (JS) First Version
****************************************************************************/

// timer.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "windows.h"
#include "iostream.h"
#include "conio.h"

// include SORCUS header files
#include "max_lib.h"
#include "x_mdd.h"

// add SORCUS maxw32.lib
#pragma comment( lib, "c:\\sorcus\\maxlib\\pc\\win32\\msvc\\maxw32" )

// some global variables
USHORT g_usBoardNo, g_usCpuSlot, g_usCpuLayer, g_usXdioSlot, g_usXdioLayer, g_usTimerInit;
MAXMODHND g_hCpu;
MAXMDDHND g_hXdioMdd;
MAXCHLHND g_hTimer;




//*****************************************************************************
// HandleError
/**
* 
* @param    pcText
* @param    Error
* @return   -
*/
//*****************************************************************************
void HandleError(CHAR* pcText, MAX_ERROR Error)
{
    CHAR acMessage[200];
    USHORT usSize = 200;

    max_clear_error();
    max_get_error_message(Error, &usSize, acMessage);
    cout << "-------------------------------------------------------------------" << endl;
    cout << "Error at        : " << pcText << endl;
    cout << "Error-Code      : " << hex << Error << dec << "hex" << endl;
    cout << "Error-Message   : " << acMessage << endl;
    cout << "-------------------------------------------------------------------" << endl;
    return;
}



//*****************************************************************************
// main
/**
*
* @param    argc
* @param    argv
* @return   int
*/
//*****************************************************************************
int main(int argc, char* argv[])
{
    USHORT usBuffer;
    MAX_ERROR Error;
    MAX_OS_INFO rcOs;
    CPS_XDIO40_A rcTimer;

    // init global variables
    g_usBoardNo = 0;
    g_usCpuSlot = 1;
    g_usCpuLayer = 0;
    g_usXdioSlot = 6;
    g_usXdioLayer = 0;
    g_usTimerInit = 10000;


    // init library
    Error = max_init_lib(0);
    
    // reset board
    Error = max_reset_board(g_usBoardNo, 20);
    if (Error != ERR_OK)
    {
        HandleError("max_reset_board", Error);
        _getch();
        return 1;
    }
    
    // connect cpu
    Error = max_connect_cpu (g_usBoardNo, g_usCpuSlot, g_usCpuLayer, &rcOs, &g_hCpu);
    if (Error != ERR_OK)
    {
        HandleError("max_connect_cpu", Error);
        _getch();
        return 1;
    }

    // load osx
    Error = max_load_osx (g_hCpu, "ROM");
    if (Error != ERR_OK)
    {
        HandleError("max_load_osx", Error);
        _getch();
        return 1;
    }

    // load mdd
    Error = max_load_mdd (g_hCpu, g_usXdioSlot, g_usXdioLayer, 0, MDD_X_DIO_40, NULL, &g_hXdioMdd);
    if (Error != ERR_OK)
    {
        HandleError("max_load_mdd", Error);
        _getch();
        return 1;
    }

    // open TIMER channel
    rcTimer.usDevice = DEVICE_TIMER;
    rcTimer.usIndexFirst = 0;
    rcTimer.usIndexLast = 0;
    rcTimer.usReadMode = IO_MODE_DIRECT;
    rcTimer.usWriteMode = IO_MODE_DIRECT;
    rcTimer.usFlags = _CP_EXCLUSIVE | 
                      _XDIO40_CLOCK_1MHZ |          // 1 MHz clock signal
                      _XDIO40_GATE_SOFTWARE |       
                      _XDIO40_OUT_DIO;              // DIO-18
    rcTimer.usMode = _XDIO40_TIMER_MODE_3 |         // square wave mode
                     _XDIO40_TIMER_MODE_BINARY;

    // open channel
    Error = max_open_channel (g_hXdioMdd, sizeof (rcTimer), &rcTimer, NULL, 0, &g_hTimer);
    if (Error != ERR_OK)
    {
        HandleError("max_open_channel", Error);
        _getch();
        return 1;
    }

    // write TIMER = 10000
    Error = max_write_channel_ushort(g_hTimer, g_usTimerInit);
    if (Error != ERR_OK)
    {
        HandleError("max_write_channel", Error);
        _getch();
        return 1;
    }

    // set GATE = 1 to activate TIMER
    usBuffer = 1;
    Error = max_channel_control(g_hTimer, CTRL_XDIO40_SET_GATE, 4, (void*)&usBuffer);
    if (Error != ERR_OK)
    {
        HandleError("max_channel_control", Error);
        _getch();
        return 1;
    }

    // while loop
    do
    {
        cout << "Next timer value (1....65535, 0=exit): ";
        cin >> g_usTimerInit;
        // write TIMER = 50000
        Error = max_write_channel_ushort(g_hTimer, g_usTimerInit);
        if (Error != ERR_OK)
        {
            HandleError("max_write_channel", Error);
            _getch();
            return 1;
        }
    }while(g_usTimerInit != 0);

    cout << "End of example" << endl;
    _getch();

    return 0;
}
