/****************************************************************************
*	TIMER.C	
*
*	PURPOSE
*		Sample Real Time program
* 		The program show handling of X-DIO-40/i module-device-driver. It shows
*		how to use the timer. We use the square wave mode to generate a 
*		square-wave signalat DOUT-18. You can modify the frequency by changing
*		parameter 20 and calling procedure 4.
*	INSTALL-PARAMETER
*		Command for SORCUS INS-File: MAXINST file="TIMER.EXE" no=900D autoinit
*	COPYRIGHT
*		(C) 2003 SORCUS Computer GmbH
*	HISTORY
*		2003-10-21 (JS) New
****************************************************************************/

#pragma option -r-

// _RTLIB_ must be defined in every real time program
#define _RTLIB_

// including header files
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"

// program version
#define VERSION         	'1'
#define REVISION        	'A'
#define PROGRAM_NUMBER		0x900D

// status of program
#define PROG_INIT			0x1
#define PROG_START		0x2
#define PROG_RUN			0x3
#define PROG_STOP			0x4

// PDT (Program Descriptor Table) data structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;		// 1
   void* MAXEXPORT start;			// 2
   void* MAXEXPORT stop;			// 3
	void* MAXEXPORT load_timer;	// 4
} rcPDT;

// parameter range of the task
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;
	MAX_ERROR    Error;				// error of program (0)
	USHORT		 usStatus;			// status of program (2)
	USHORT       usSlot;				// slot of module (4)
	USHORT		 usLayer;			// layer of module (6)
   MAXMODHND    hMyself;        	// handle for the CPU (8)
	MAXCHLHND    hTimer;				// handle to TIMER channel (12)
	MAXMDDHND    hMdd;            // handle for the MDD (16)
	USHORT       usTimerInit;		// initial value of timer (20)
	USHORT		 usTimerValue;		// value of timer (22)
	MAX_OS_INFO  rcOsX;				// OsX (24)
} rcParameter;

// Prototypes
void MAXEXPORT MainProc (void);
void MAXEXPORT AutoInit (void);
void MAXEXPORT Start (void);
void MAXEXPORT Stop (void);
void MAXEXPORT LoadTimer(void);


/*************************************************************************
*			AutoInit
*************************************************************************/
void MAXEXPORT AutoInit(void)
{
	max_entry_proc();

   // init library
	rcParameter.Error = max_init_lib(0);
   if (rcParameter.Error == ERR_OK) 
	{
      // get a handle for the X-MAX-1
      rcParameter.Error = max_connect_cpu (0, MAX_MYSELF, 0, &rcParameter.rcOsX, &rcParameter.hMyself);
   }

	// set slot and layer of X-DIO-40
	rcParameter.usSlot = 6;
	rcParameter.usLayer = 0;

	// status
	rcParameter.usStatus = PROG_INIT;

	// timer init value
	rcParameter.usTimerInit = 10000;

   max_exit_proc();
};



/*************************************************************************
*			Start
*************************************************************************/
void MAXEXPORT Start(void)
{
   USHORT         i, usBuffer;
   CPS_XDIO40_A   rcTimer;

   max_entry_proc();

	// check error
	if (rcParameter.Error == ERR_OK)
	{
		// get a handle for the MDD
		rcParameter.Error = max_load_mdd (rcParameter.hMyself, rcParameter.usSlot, rcParameter.usLayer, 0, MDD_X_DIO_40, NULL, &rcParameter.hMdd);
	   if (rcParameter.Error == ERR_OK) 
		{
			// fill in CPS for TIMER channel
			rcTimer.usDevice      = DEVICE_TIMER;
			rcTimer.usIndexFirst  = 0;
			rcTimer.usIndexLast   = 0;
			rcTimer.usReadMode    = IO_MODE_DIRECT;
			rcTimer.usWriteMode   = IO_MODE_DIRECT;
			rcTimer.usFlags       = _CP_EXCLUSIVE | 
										   _XDIO40_CLOCK_1MHZ |			// 1 MHz clock signal
										   _XDIO40_GATE_SOFTWARE |		
										   _XDIO40_OUT_DIO;				// DIO-18
			rcTimer.usMode        = _XDIO40_TIMER_MODE_3 | 		// square wave mode
											_XDIO40_TIMER_MODE_BINARY;

			// open channel
		   rcParameter.Error = max_open_channel (rcParameter.hMdd, sizeof (rcTimer), &rcTimer, NULL, 0, &rcParameter.hTimer);
		
			// write TIMER
			rcParameter.Error = max_write_channel_ushort(rcParameter.hTimer, 	rcParameter.usTimerInit);

			// set GATE = 1 to activate TIMER
			usBuffer = 1;
			rcParameter.Error = max_channel_control(rcParameter.hTimer, CTRL_XDIO40_SET_GATE, 4, (void*)&usBuffer);

	     	if (rcParameter.Error == ERR_OK)
		  	{
				// activate task
				rcParameter.Error = max_wakeup_ni_task(rcParameter.hMyself, rcParameter.rcTDT.usTaskNo, 1);
		  	}
		}
	}

	// status
	rcParameter.usStatus = PROG_START;

   max_exit_proc();
}



/*************************************************************************
*			MainProc
*************************************************************************/
void MAXEXPORT MainProc(void)
{
   UCHAR ucLed, ucOutData;
	MAX_ERROR Error;

   max_entry_proc();

   if (rcParameter.Error == ERR_OK) 
	{
		// read TIMER channel
		rcParameter.Error = max_read_channel_ushort(rcParameter.hTimer, &rcParameter.usTimerValue);
   }

	// status
	rcParameter.usStatus = PROG_RUN;

   max_exit_proc();
}



/*************************************************************************
*			Stop
*************************************************************************/
void MAXEXPORT Stop(void)
{
   max_entry_proc();

	// deactivate task
   rcParameter.Error = max_sleep_task (rcParameter.hMyself, rcParameter.rcTDT.usTaskNo);

	// close channels
	rcParameter.Error = max_close_channel(&rcParameter.hTimer);

	// status
	rcParameter.usStatus = PROG_STOP;

   max_exit_proc();
}



/*************************************************************************
*			LoadTimer
*************************************************************************/
void MAXEXPORT LoadTimer(void)
{
	max_entry_proc();

   // load new timer value
   if (rcParameter.Error == ERR_OK) 
	{
		rcParameter.Error = max_write_channel_ushort(rcParameter.hTimer, rcParameter.usTimerInit);
	}

   max_exit_proc();
};



/*************************************************************************
*
*************************************************************************/
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_NI_TASK + MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = 0;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = PROGRAM_NUMBER;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&rcParameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(rcParameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;
	rcPDT.load_timer       = LoadTimer;
   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
