/****************************************************************************/
// Sample program to demonstrate the usage of the MDD for X-SCC-2. 
// 
// This sample shows the usage of the register channels, the configuration channel
// and the interrupt channel.
// 
// Usage:
// 1. Select the Board ID of the board where the X-SCC-2 is pluged, the slot of the module
//    and the channel (0 or 1) you want to use for communication.
// 2. Press Reset/download button.
// 3. Select the desired properties for the control channel and open the channel.
// 4. If desired select the properties for the interrupt channel and open the channel. 
// 5. The contents of all the SCC's WR-register are displayed on the left (in hex)
// 6. The contents of all the SCC's RR-register are displayed in a row in the middle (in hex)
// 7. The button 'Config async.' writes a sample configuration into the SCC, 
//    for which the communication parameters on the left of the button are used.
// 8. The button 'Send' transmits a few characters by writing the corresponding register.
// 9.  The value of any WR register may be changed by modifying the value in the 
//    corresponding edit field and pressing the corresponding 'Set' button. 
//
//    Refer to the module manual and the Zilog SCC user's manual.
//
// Copyright SORCUS Computer GmbH 2004
/****************************************************************************/

#include "stdafx.h"
#include "testxscc.h"
#include "testxsccDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

MAXMODHND   hXMAX;
MAXMDDHND   hMdd;
MAXCHLHND	hRR[16], hCtrl, hWR[16], hWR7prime, hInt;

// choose platform where the MDD should be loaded
#define OSX_MDD		 // use X-MAX-1
//#define SYS_MDD          // use WIN32 kernel mode MDD   

/////////////////////////////////////////////////////////////////////////////
// CTestxsccDlg Dialogfeld

CTestxsccDlg::CTestxsccDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTestxsccDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTestxsccDlg)
	m_IntOnBRK = FALSE;
	m_IntOnCTS = FALSE;
	m_IntOnDCD = FALSE;
	m_DTRInternal = FALSE;
	m_IntOnParity = FALSE;
	m_Rbus = FALSE;
	m_IntOnRCA = FALSE;
	m_RTS_RTXC = FALSE;
	m_RTSCTSOut = FALSE;
	m_Slow = FALSE;
	m_IntOnSpecCond = FALSE;
	m_Stop = FALSE;
	m_IntOnSync = FALSE;
	m_IntOnTBE = FALSE;
	m_IntOnTMTUR = FALSE;
	m_IntOnZero = FALSE;
	m_WR0 = _T("");
	m_WR1 = _T("");
	m_WR10 = _T("");
	m_WR11 = _T("");
	m_WR12 = _T("");
	m_WR13 = _T("");
	m_WR14 = _T("");
	m_WR15 = _T("");
	m_WR2 = _T("");
	m_WR3 = _T("");
	m_WR4 = _T("");
	m_WR5 = _T("");
	m_WR6 = _T("");
	m_WR7 = _T("");
	m_WR7PR = _T("");
	m_WR8 = _T("");
	m_WR9 = _T("");
	m_RR = _T("");
	m_Baudrate = 0;
	m_usBoard = 0;
	m_usChannel = 0;
	m_usSlot = 0;
	//}}AFX_DATA_INIT
	// Beachten Sie, dass LoadIcon unter Win32 keinen nachfolgenden DestroyIcon-Aufruf bentigt
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTestxsccDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTestxsccDlg)
	DDX_Control(pDX, IDC_COMBO_STOPBIT, m_Stopbit);
	DDX_Control(pDX, IDC_COMBO_PARITY, m_Parity);
	DDX_Control(pDX, IDC_COMBO_BPC, m_BPC);
	DDX_Control(pDX, IDC_COMBO_TRXC, m_TRxC);
	DDX_Control(pDX, IDC_COMBO_RTXC, m_RTxC);
	DDX_Control(pDX, IDC_COMBO_PHYS_TYPE, m_Interface);
	DDX_Check(pDX, IDC_CHECK_BRK, m_IntOnBRK);
	DDX_Check(pDX, IDC_CHECK_CTS, m_IntOnCTS);
	DDX_Check(pDX, IDC_CHECK_DCD, m_IntOnDCD);
	DDX_Check(pDX, IDC_CHECK_DTRINT, m_DTRInternal);
	DDX_Check(pDX, IDC_CHECK_PAR, m_IntOnParity);
	DDX_Check(pDX, IDC_CHECK_RBUS, m_Rbus);
	DDX_Check(pDX, IDC_CHECK_RCA, m_IntOnRCA);
	DDX_Check(pDX, IDC_CHECK_RTS_TRXC, m_RTS_RTXC);
	DDX_Check(pDX, IDC_CHECK_RTSCTSOUT, m_RTSCTSOut);
	DDX_Check(pDX, IDC_CHECK_SLOW, m_Slow);
	DDX_Check(pDX, IDC_CHECK_SPCND, m_IntOnSpecCond);
	DDX_Check(pDX, IDC_CHECK_STOP, m_Stop);
	DDX_Check(pDX, IDC_CHECK_SYNC, m_IntOnSync);
	DDX_Check(pDX, IDC_CHECK_TBE, m_IntOnTBE);
	DDX_Check(pDX, IDC_CHECK_TMTUR, m_IntOnTMTUR);
	DDX_Check(pDX, IDC_CHECK_ZERO, m_IntOnZero);
	DDX_Text(pDX, IDC_EDIT_WR0, m_WR0);
	DDX_Text(pDX, IDC_EDIT_WR1, m_WR1);
	DDX_Text(pDX, IDC_EDIT_WR10, m_WR10);
	DDX_Text(pDX, IDC_EDIT_WR11, m_WR11);
	DDX_Text(pDX, IDC_EDIT_WR12, m_WR12);
	DDX_Text(pDX, IDC_EDIT_WR13, m_WR13);
	DDX_Text(pDX, IDC_EDIT_WR14, m_WR14);
	DDX_Text(pDX, IDC_EDIT_WR15, m_WR15);
	DDX_Text(pDX, IDC_EDIT_WR2, m_WR2);
	DDX_Text(pDX, IDC_EDIT_WR3, m_WR3);
	DDX_Text(pDX, IDC_EDIT_WR4, m_WR4);
	DDX_Text(pDX, IDC_EDIT_WR5, m_WR5);
	DDX_Text(pDX, IDC_EDIT_WR6, m_WR6);
	DDX_Text(pDX, IDC_EDIT_WR7, m_WR7);
	DDX_Text(pDX, IDC_EDIT_WR7P, m_WR7PR);
	DDX_Text(pDX, IDC_EDIT_WR8, m_WR8);
	DDX_Text(pDX, IDC_EDIT_WR9, m_WR9);
	DDX_Text(pDX, IDC_EDIT_RR, m_RR);
	DDX_Text(pDX, IDC_EDIT_BAUD, m_Baudrate);
	DDX_Text(pDX, IDC_EDIT_BOARD, m_usBoard);
	DDX_Text(pDX, IDC_EDIT_CHANNEL, m_usChannel);
	DDX_Text(pDX, IDC_EDIT_SLOT, m_usSlot);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTestxsccDlg, CDialog)
	//{{AFX_MSG_MAP(CTestxsccDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_CTRL, OnButtonCtrl)
	ON_BN_CLICKED(IDC_BUTTON_INT, OnButtonInt)
	ON_BN_CLICKED(IDC_BUTTON_WR0, OnButtonWr0)
	ON_BN_CLICKED(IDC_BUTTON_WR1, OnButtonWr1)
	ON_BN_CLICKED(IDC_BUTTON_WR2, OnButtonWr2)
	ON_BN_CLICKED(IDC_BUTTON_WR3, OnButtonWr3)
	ON_BN_CLICKED(IDC_BUTTON_WR4, OnButtonWr4)
	ON_BN_CLICKED(IDC_BUTTON_WR5, OnButtonWr5)
	ON_BN_CLICKED(IDC_BUTTON_WR6, OnButtonWr6)
	ON_BN_CLICKED(IDC_BUTTON_WR7, OnButtonWr7)
	ON_BN_CLICKED(IDC_BUTTON_WR8, OnButtonWr8)
	ON_BN_CLICKED(IDC_BUTTON_WR9, OnButtonWr9)
	ON_BN_CLICKED(IDC_BUTTON_WR10, OnButtonWr10)
	ON_BN_CLICKED(IDC_BUTTON_WR11, OnButtonWr11)
	ON_BN_CLICKED(IDC_BUTTON_WR12, OnButtonWr12)
	ON_BN_CLICKED(IDC_BUTTON_WR13, OnButtonWr13)
	ON_BN_CLICKED(IDC_BUTTON_WR14, OnButtonWr14)
	ON_BN_CLICKED(IDC_BUTTON_WR15, OnButtonWr15)
	ON_BN_CLICKED(IDC_BUTTON_WR7P, OnButtonWr7p)
	ON_BN_CLICKED(IDC_CHECK_STOP, OnCheckStop)
	ON_BN_CLICKED(IDC_CHECK_SLOW, OnCheckSlow)
	ON_BN_CLICKED(IDC_CHECK_RBUS, OnCheckRbus)
	ON_BN_CLICKED(IDC_CHECK_RTSCTSOUT, OnCheckRtsctsout)
	ON_BN_CLICKED(IDC_BUTTON_ASYNC, OnButtonAsync)
	ON_BN_CLICKED(IDC_BUTTON_TMT, OnButtonTmt)
	ON_BN_CLICKED(IDC_BUTTON_START, OnButtonStart)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTestxsccDlg Nachrichten-Handler

/////////////////////////////////////////////////////////////////////////////
// OnInitDialog : Init Lib
BOOL CTestxsccDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

	// Symbol fr dieses Dialogfeld festlegen. Wird automatisch erledigt
	//  wenn das Hauptfenster der Anwendung kein Dialogfeld ist
	SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
	SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden
	
	max_init_lib(0);

    m_usBoard = 1;
    m_usSlot = 6;           
    m_usChannel = 0;
    m_TRxC.SetCurSel(3);
    m_Interface.SetCurSel(0);
    m_RTxC.SetCurSel(3);
    m_IntOnCTS = TRUE;
    m_IntOnDCD = TRUE;
    m_IntOnParity = TRUE;
    m_IntOnRCA = TRUE;
    m_IntOnSpecCond = TRUE;
    m_IntOnTBE = TRUE;
	UpdateData(FALSE); 
    
    return TRUE;  
}

/////////////////////////////////////////////////////////////////////////////
// OnPaint
void CTestxsccDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // Gertekontext fr Zeichnen

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Symbol in Client-Rechteck zentrieren
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Symbol zeichnen
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CTestxsccDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CTestxsccDlg::OnOK() 
{
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonStart: Reset board and load MDD
void CTestxsccDlg::OnButtonStart() 
{
    MAX_ERROR   result;
    MAX_OS_INFO rcOs;
    MAX_PGM_TYPE   rcMdd = {0};
    CString     sText;
	MAX_VERSION	Version;
    ULONG       ulDate;
	char	    szVersion[40], szDate[40], szMsg[80];
	MAX_MODULINFO_TYPE rcModulInfo;

	UpdateData(TRUE); 

    // reset board
	result = max_reset_board(m_usBoard, 30);
	if (result) 
    {
		AfxMessageBox("Error reseting board");
		exit (0);
	}

    // Get CPU handle
#ifdef OSX_MDD	
    result = max_connect_cpu (m_usBoard, MAX_AUTO_SLOT, 0, &rcOs, &hXMAX);
	if (result) 
    {
        sText.Format("Error %X connecting cpu", result);
		AfxMessageBox(sText);
		exit (0);
	}
    if (rcOs.usType == MAX_MINI_OSX)
    {
        result = max_load_osx (hXMAX, "ROM");
	    if (result)
	    {
            sText.Format("Error %X loading osx", result);
		    AfxMessageBox(sText);
		    exit (0);
	    }
    }
#else
    result = max_connect_cpu (m_usBoard, 0, 0, &rcOs, &hXMAX);
	if (result) 
    {
        sText.Format("Error %X connecting cpu", result);
		AfxMessageBox(sText);
		exit (0);
	}
#endif

    // load MDD for the X-SCC-2
    result = max_load_mdd (hXMAX, m_usSlot, 0, 0, MDD_X_SCC_2, NULL, &hMdd);
	if (result)
	{
        sText.Format("Error %X loading mdd", result);
		AfxMessageBox(sText);
		exit (0);
	}

#ifndef DLL_MDD
	result = max_get_module_info(m_usBoard, m_usSlot, 0, &rcModulInfo);
    max_get_mdd_version(hMdd, &Version, &ulDate);
    max_get_version_string(Version, ulDate, szVersion, szDate);
    sprintf (szMsg, "MDD (Version %s) loaded", szVersion);
	AfxMessageBox(szMsg);
#endif
    
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonCtrl Open Control and Register channels
void CTestxsccDlg::OnButtonCtrl() 
{
	CPS_XSCC2    cps;
	MAX_ERROR	Error;
    CString     sText;
    USHORT      i;

	UpdateData(TRUE); 

    cps.usVersion = 0;
    cps.usChannel = 0;      
    cps.usDevice = DEVICE_CTRL;
    cps.usIndexFirst = XSCC_CTRL_PHYS_IFC;
    cps.usIndexLast = cps.usIndexFirst;
    cps.usReadMode = IO_MODE_RAM;
    cps.usWriteMode = IO_MODE_DIRECT;
    cps.usMode = 0;
    cps.usFlags = _CP_EXCLUSIVE;
    if (m_DTRInternal)
        cps.usFlags |= XSCC_DTR_INTERNAL; 
    if (m_RTS_RTXC)
        cps.usFlags |= XSCC_RTS_FROM_TRXC; 
    switch (m_Interface.GetCurSel())
    {
    case 0: cps.usType = COM_RS232;
            break;
    case 1: cps.usType = COM_RS422;
            break;
    case 2: cps.usType = COM_RS485_HALF_DUPLEX;
            break;
    case 3: cps.usType = COM_RS485_FULL_DUPLEX;
            break;
    case 4: cps.usType = COM_CURRENT_LOOP;
            break;
    }
    switch (m_RTxC.GetCurSel())
    {
    case 0: cps.usRTxC = XSCC_SEL_CLKIO;
            break;
    case 1: cps.usRTxC = XSCC_SEL_RI;
            break;
    case 2: cps.usRTxC = XSCC_SEL_CTS;
            break;
    case 3: cps.usRTxC = XSCC_SEL_PCLK;
            break;
    }
    switch (m_TRxC.GetCurSel())
    {
    case 0: cps.usTRxC = XSCC_SEL_CLKIO;
            break;
    case 1: cps.usTRxC = XSCC_SEL_RI;
            break;
    case 2: cps.usTRxC = XSCC_SEL_CTS;
            break;
    case 3: cps.usTRxC = XSCC_SEL_NC;
            break;
    }
	Error = max_open_channel(hMdd, sizeof(cps), &cps, NULL, 0, &hCtrl);
	if (Error)
    {
        sText.Format("Error %X opening control channel", Error);
		AfxMessageBox(sText);
#ifndef DLL_MDD
        max_clear_error();
#endif
        return;
    }

    cps.usDevice = DEVICE_REGISTER;
    cps.usFlags = _CP_EXCLUSIVE;
    cps.usMode = 0;
    
    for (i=0; i<16; i++)
    {
        cps.usIndexFirst = i;
        cps.usIndexLast = cps.usIndexFirst;
        cps.usReadMode = IO_MODE_RAM;
        cps.usWriteMode = IO_MODE_DIRECT;
        Error = max_open_channel(hMdd, sizeof(cps), &cps, NULL, 0, &hWR[i]);
        if (Error)
        {
            sText.Format("Error %X opening channel WR%d", Error, i);
            AfxMessageBox(sText);
#ifndef DLL_MDD
            max_clear_error();
#endif
        }
        cps.usReadMode = IO_MODE_DIRECT;
        cps.usWriteMode = 0;
        Error = max_open_channel(hMdd, sizeof(cps), &cps, NULL, 0, &hRR[i]);
        if (Error)
        {
            sText.Format("Error %X opening channel RR%d", Error, i);
            AfxMessageBox(sText);
#ifndef DLL_MDD
            max_clear_error();
#endif
        }
    }

    cps.usIndexFirst = XSCC_WR7_PRIME;;
    cps.usIndexLast  = cps.usIndexFirst;
    cps.usReadMode   = IO_MODE_RAM;
    cps.usWriteMode  = IO_MODE_DIRECT;
    Error = max_open_channel(hMdd, sizeof(cps), &cps, NULL, 0, &hWR7prime);
    if (Error)
    {
        sText.Format("Error %X opening channel WR%d", Error, i);
        AfxMessageBox(sText);
#ifndef DLL_MDD
        max_clear_error();
#endif
    }
    else
        UpdateRegister ();

    m_Baudrate = 9600;
    m_BPC.SetCurSel(3);
    m_Stopbit.SetCurSel(1);
    m_Parity.SetCurSel(0);
	UpdateData(FALSE); 
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonInt Open Interrupt channel
void CTestxsccDlg::OnButtonInt() 
{
	CPS_XSCC2    cps;
	MAX_ERROR	Error;
    CString     sText;

	UpdateData(TRUE); 

    cps.usVersion = 0;
    cps.usChannel = 0;      
    cps.usDevice = DEVICE_HW_INT;
    cps.usFlags = _CP_EXCLUSIVE;
    cps.usIndexFirst = 0;
    cps.usIndexLast = cps.usIndexFirst;
    cps.usReadMode = 0;
    cps.usWriteMode = 0;
    cps.usMode = 0;
    if (m_IntOnBRK)
        cps.usMode |= XSCC_INT_BRK;
    if (m_IntOnRCA)
        cps.usMode |= XSCC_INT_RCA;
    if (m_IntOnSpecCond)
        cps.usMode |= XSCC_INT_SPEC_COND;
    if (m_IntOnTBE)
        cps.usMode |= XSCC_INT_TBE;
    if (m_IntOnParity)
        cps.usMode |= XSCC_INT_PARITY_ERR;
    if (m_IntOnTMTUR)
        cps.usMode |= XSCC_INT_TMTUR_EOM;
    if (m_IntOnCTS)
        cps.usMode |= XSCC_INT_CTS;
    if (m_IntOnDCD)
        cps.usMode |= XSCC_INT_DCD;
    if (m_IntOnSync)
        cps.usMode |= XSCC_INT_SYNC;
    if (m_IntOnZero)
        cps.usMode |= XSCC_INT_ZERO;
	Error = max_open_channel(hMdd, sizeof(cps), &cps, MyCallback, 0, &hInt);
	if (Error)
    {
        sText.Format("Error %X opening control channel", Error);
		AfxMessageBox(sText);
#ifndef DLL_MDD
        max_clear_error();
#endif
    }
    UpdateRegister ();

}

/////////////////////////////////////////////////////////////////////////////
// UpdateRegister : display values of all SCCs Read and Write register
void CTestxsccDlg::UpdateRegister()
{
    UCHAR ucData, i;
    CString *apsRegCont[16] = {&m_WR0, &m_WR1, &m_WR2, &m_WR3, &m_WR4, &m_WR5, &m_WR6, &m_WR7,
                               &m_WR8, &m_WR9, &m_WR10, &m_WR11, &m_WR12, &m_WR13, &m_WR14, &m_WR15};
    CString sText;
    MAX_ERROR Err;

    for (i=0; i<16; i++)
    {
        Err = max_read_channel_uchar(hWR[i], &ucData);
        apsRegCont[i]->Format("%02X", ucData);
    }
    max_read_channel_uchar(hWR7prime, &ucData);
    m_WR7PR.Format("%02X", ucData);

    m_RR = "";
    for (i=0; i<16; i++)
    {
        Err = max_read_channel_uchar(hRR[i], &ucData);
        sText.Format("%02X ", ucData);
        m_RR = m_RR + sText;
    }
	UpdateData(FALSE); 
}


/////////////////////////////////////////////////////////////////////////////
// SetWR : write data into a WR register
void CTestxsccDlg::SetWR(UCHAR ucReg, CString sData)
{
    UCHAR ucData;

    sscanf(sData, "%X", &ucData);
    max_write_channel_uchar(hWR[ucReg], ucData);
    UpdateRegister();
}

/////////////////////////////////////////////////////////////////////////////
// MyCallback : MDD callback channel
MAX_CALLBACK CTestxsccDlg::MyCallback(MAXCHLHND hChannel, ULONG ulParam, ULONG ulSize, void *pData)
{
    CString sText;
    USHORT  usEvent = *(USHORT*)pData;      // first word is the event
    USHORT  usData = *((USHORT*)pData+1);   // second word contains interrupt specific additional information

    if (usEvent == XSCC_INT_RCA)
    {
        sText.Format("Event %X : Receive character available: %X", usEvent, usData);
    }

    if (usEvent == XSCC_INT_TBE)
    {
        sText.Format("Event %X : Transmit buffer empty", usEvent);
    }
    
    if (usEvent & XSCC_INT_SPEC_COND) 
    {
        sText.Format("Event %X : Spec Condition (RR1=%X)", usEvent, usData);
        if (usEvent & XSCC_INT_PARITY_ERR) 
            sText = sText + " Parity error";
        if (usEvent & XSCC_INT_SPEC_FRAME) 
            sText = sText + " Frame error";
        if (usEvent & XSCC_INT_SPEC_EOF) 
            sText = sText + " EOF";
        if (usEvent & XSCC_INT_SPEC_OVERRUN) 
            sText = sText + " Overrun";
    }
    if (usEvent & (XSCC_INT_BRK + XSCC_INT_TMTUR_EOM + XSCC_INT_CTS + 
                   XSCC_INT_SYNC +  XSCC_INT_DCD + XSCC_INT_ZERO) )
    {
        sText.Format("Event %X : External/Status change (RR0=%X)", usEvent, usData);
        if (usEvent & XSCC_INT_BRK) 
            sText = sText + " Break";
        if (usEvent & XSCC_INT_TMTUR_EOM) 
            sText = sText + " TmtUR/EOM";
        if (usEvent & XSCC_INT_CTS) 
            sText = sText + " CTS";
        if (usEvent & XSCC_INT_SYNC) 
            sText = sText + " Sync";
        if (usEvent & XSCC_INT_DCD) 
            sText = sText + " DCD";
        if (usEvent & XSCC_INT_ZERO) 
            sText = sText + " Zero";
    }
    AfxMessageBox(sText);
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonWrx : Handlers for all WR-Set buttons (WR0..WR15)
void CTestxsccDlg::OnButtonWr0() 
{
	UpdateData(TRUE); 
    SetWR(0, m_WR0);	
}

void CTestxsccDlg::OnButtonWr1() 
{
	UpdateData(TRUE); 
    SetWR(1, m_WR1);	
}

void CTestxsccDlg::OnButtonWr2() 
{
	UpdateData(TRUE); 
    SetWR(2, m_WR2);	
}

void CTestxsccDlg::OnButtonWr3() 
{
	UpdateData(TRUE); 
    SetWR(3, m_WR3);	
}

void CTestxsccDlg::OnButtonWr4() 
{
	UpdateData(TRUE); 
    SetWR(4, m_WR4);	
}

void CTestxsccDlg::OnButtonWr5() 
{
	UpdateData(TRUE); 
    SetWR(5, m_WR5);	
}

void CTestxsccDlg::OnButtonWr6() 
{
	UpdateData(TRUE); 
    SetWR(6, m_WR6);	
}

void CTestxsccDlg::OnButtonWr7() 
{
	UpdateData(TRUE); 
    SetWR(7, m_WR7);	
}

void CTestxsccDlg::OnButtonWr8() 
{
	UpdateData(TRUE); 
    SetWR(8, m_WR8);	
}

void CTestxsccDlg::OnButtonWr9() 
{
	UpdateData(TRUE); 
    SetWR(9, m_WR9);	
}

void CTestxsccDlg::OnButtonWr10() 
{
	UpdateData(TRUE); 
    SetWR(10, m_WR10);	
}

void CTestxsccDlg::OnButtonWr11() 
{
	UpdateData(TRUE); 
    SetWR(11, m_WR11);	
}

void CTestxsccDlg::OnButtonWr12() 
{
	UpdateData(TRUE); 
    SetWR(12, m_WR12);	
}

void CTestxsccDlg::OnButtonWr13() 
{
	UpdateData(TRUE); 
    SetWR(13, m_WR13);	
}

void CTestxsccDlg::OnButtonWr14() 
{
	UpdateData(TRUE); 
    SetWR(14, m_WR14);	
}

void CTestxsccDlg::OnButtonWr15() 
{
	UpdateData(TRUE); 
    SetWR(15, m_WR15);	
}


/////////////////////////////////////////////////////////////////////////////
// OnButtonWr7P : Handlers for WR7Prime-Set button 
void CTestxsccDlg::OnButtonWr7p() 
{
    UCHAR ucData;

	UpdateData(TRUE); 
    sscanf(m_WR7PR, "%X", &ucData);
    max_write_channel_uchar(hWR7prime, ucData);
    UpdateRegister();
}

/////////////////////////////////////////////////////////////////////////////
// OnCheckStop: CMD_START/STOP for Control channel
void CTestxsccDlg::OnCheckStop() 
{
    // not implemented yet	
}

/////////////////////////////////////////////////////////////////////////////
// OnCheckSlow: CMD_XSCC_SLOW/FAST for Control channel
void CTestxsccDlg::OnCheckSlow() 
{
    // not implemented yet	
}

/////////////////////////////////////////////////////////////////////////////
// OnCheckRbus: CMD_XSCC_RBUS_ON/OFF for Control channel
void CTestxsccDlg::OnCheckRbus() 
{
    // not implemented yet	
}

/////////////////////////////////////////////////////////////////////////////
// OnCheckRtsctsout: CMD_DIR_INPUT/OUTPUT for Control channel
void CTestxsccDlg::OnCheckRtsctsout() 
{
    // not implemented yet	
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonAsync: programm ESCC to asynchronous mode 
// this shows the usage of the register channels
void CTestxsccDlg::OnButtonAsync() 
{
    UCHAR ucData;
    ULONG TC, divider;

	UpdateData(TRUE); 
    switch(m_BPC.GetCurSel())
    {
    case 0: 
	    max_write_channel_uchar(hWR[3], XSCC_RX_5_BPC + XSCC_RX_ENABLE);
	    max_write_channel_uchar(hWR[5], XSCC_TX_5_BPC);
        break;
    case 1: 
	    max_write_channel_uchar(hWR[3], XSCC_RX_6_BPC + XSCC_RX_ENABLE);
	    max_write_channel_uchar(hWR[5], XSCC_TX_6_BPC);
        break;
    case 2: 
	    max_write_channel_uchar(hWR[3], XSCC_RX_7_BPC + XSCC_RX_ENABLE);
	    max_write_channel_uchar(hWR[5], XSCC_TX_7_BPC);
        break;
    case 3: 
	    max_write_channel_uchar(hWR[3], XSCC_RX_8_BPC + XSCC_RX_ENABLE);
	    max_write_channel_uchar(hWR[5], XSCC_TX_8_BPC);
        break;
    }
    switch(m_Parity.GetCurSel())
    {
    case 0: ucData = 0;
        break;
    case 1: ucData = XSCC_PARITY_ENABLE;
        break;
    case 2: ucData = XSCC_PARITY_ENABLE + XSCC_EVEN_PARITY;
        break;
    }
    switch(m_Stopbit.GetCurSel())
    {
    case 1: ucData |= XSCC_STOP_BIT_1;
        break;
    case 2: ucData |= XSCC_STOP_BIT_1_5;
        break;
    case 3: ucData |= XSCC_STOP_BIT_2;
        break;
    }
    ucData |= XSCC_X16_CLOCK_MODE;
	max_write_channel_uchar(hWR[4], ucData);
	max_write_channel_uchar(hWR[10], XSCC_BIT_CODE_NRZ);
	max_write_channel_uchar(hWR[11], XSCC_TMTCLK_BRG + XSCC_RCVCLK_BRG);
	// program baudrate generator time constant
	// TC = 14745600Hz/(2x16x9600) - 2 = 46 (s. SCC manual baud rate generator)
    divider = m_Baudrate * 32; 
    TC = 14745600 / divider;
    TC -= 2;
	max_write_channel_uchar(hWR[12], (UCHAR)TC);
    TC >>=8;
	max_write_channel_uchar(hWR[13], (UCHAR)TC);
	// select and enable baud rate generator in 2 steps:
	max_write_channel_uchar(hWR[14], XSCC_BRG_SEL_PCLK);
	max_write_channel_uchar(hWR[14], XSCC_BRG_SEL_PCLK + XSCC_BRG_ENABLE);
    UpdateRegister();
}

/////////////////////////////////////////////////////////////////////////////
// OnButtonTmt: transmit some bytes
void CTestxsccDlg::OnButtonTmt() 
{
    USHORT i, usTimeout;
    UCHAR aucData[4] = {0x31, 0x42, 0x53, 0x64}, ucStatus;

	max_write_channel_uchar(hWR[5], XSCC_TX_8_BPC + XSCC_TX_ENABLE);
	for (i=0; i<sizeof(aucData); i++)
  	{	
		// in this sample we write one byte into the transmit buffer
		// then we wait for the transmit buffer to become empty 
		// before writing the next byte 
		// this can be optimized e.g. by using the FIFO 
		max_write_channel_uchar(hWR[8], aucData[i]);
		usTimeout = 1000;
		do 
	  	{	// poll the ALL SENT bit to be sure that the data was transmitted
			max_read_channel_uchar(hRR[1], &ucStatus);
			usTimeout--;	
	  	}
		while	(((ucStatus & XSCC_ALL_SENT) == 0) && (usTimeout > 0));
	}
  	// disable transmitter
	max_write_channel_uchar(hWR[5], XSCC_TX_8_BPC);
    UpdateRegister();
}

