/****************************************************************************/
// Sample Real Time program 
//
// The program opens channel to register channels and trasnmits data.
//
// Program Number 9011h
//
// (C)2000 SORCUS Computer GmbH
/****************************************************************************/

#pragma option -r-

#define _RTLIB_                        // must be defined in every real time
// program before including max_lib.h
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"

#define PROGRAMM_NUMBER 0x9011
#define VERSION         '1'		//version of program
#define REVISION        'A'		//revision of program

// PDT structure
struct pdt_type
{
	MAX_PDT_HEAD    rcHead;
	void* MAXEXPORT main_proc;
	void* MAXEXPORT auto_init;
	void* MAXEXPORT start;
	void* MAXEXPORT stop;
   void  MAXEXPORT (*send) (USHORT *pusInsize, UCHAR *pucData,
                            USHORT *pusOutsize, USHORT *pDataOut);
} rcPDT;

// parameter range structure
struct parameter_type
{
	MAX_TDT_TYPE rcTDT;                 // always before the first parameter !
	// offset 0
	MAX_ERROR    Error;                 // Errorcode
	USHORT		 usFunc;						// function number that caused the error

	USHORT		 usSlotXSCC;				// slot number of the X-SCC-2
	USHORT		 usLayerXSCC;				// layer number of the X-SCC-2
	USHORT		 usChannel;					// SCC channel (0 or 1)
	UCHAR			 ucLastReceived;
	UCHAR			 ucDummy;

	// counters for interrupt events
	ULONG		    ulTBEIntCount;
	ULONG		    ulRCAIntCount;
	ULONG		    ulParityIntCount;
	ULONG		    ulBRKIntCount;
	ULONG		    ulTMTURIntCount;
	ULONG		    ulCTSIntCount;
	ULONG		    ulDCDIntCount;
	ULONG		    ulSYNCIntCount;
	ULONG		    ulZEROIntCount;
	ULONG		    ulEOFCount;
	ULONG		    ulFRAMEIntCount;
	ULONG		    ulOVRIntCount;

	// handles: in the parameter range for easy bug check 
	// for faster operation it may be global variables
	MAXMODHND    hMdd;               	// Handle for the SCC MDD
	// handles for the ESCC register (only the significant register)
	MAXCHLHND	 hWR0, hWR1, hWR2, hWR3, hWR4, hWR5, hWR6, hWR7, hWR7Prime;
	MAXCHLHND	 hWR8, hWR9, hWR10, hWR11, hWR12, hWR13, hWR14, hWR15;
	MAXCHLHND	 hRR0, hRR1, hRR2, hRR6, hRR7, hRR10;
	MAXCHLHND	 hInterface;				// handle for the configuration channel
	MAXCHLHND	 hInterrupt;				// handle for the interrupt channel

	MAXMODHND    hMyself;               // Handle for the CPU module
	MAX_OS_INFO  rcOsX;                 // Version info about OsX
} parameter;


/****************************************************************************/
// callback function for SCC interrupts
// this is an sample that decodes all possible interruzpt events
// It is enough to check only those events that were enabled 
// when openning the interrupt channel.
#pragma argsused
MAX_CALLBACK ISR(MAXCHLHND handle, ULONG par, ULONG size, void *pData)
{
	USHORT usEvent = *(USHORT*)pData;		 // first word is the event
   USHORT usData = *((USHORT*)pData+1);   // second word contains interrupt specific additional information

 	if (usEvent & XSCC_INT_RCA)
   {	// receive character available
		parameter.ulRCAIntCount++;
  	}
	if (usEvent & (XSCC_INT_SPEC_COND+XSCC_INT_PARITY_ERR))
   {	// special conditions:
		if (usEvent & XSCC_INT_SPEC_EOF)
		{
			parameter.ulEOFCount++;
	  	}
		if (usEvent & XSCC_INT_SPEC_FRAME)
		{
			parameter.ulFRAMEIntCount++;
	  	}
		if (usEvent & XSCC_INT_SPEC_OVERRUN)
		{
			parameter.ulOVRIntCount++;
	  	}
		if (usEvent & XSCC_INT_PARITY_ERR)
		{
			parameter.ulParityIntCount++;
	  	}
  	}
	if (usEvent & XSCC_INT_TBE)
   {	// transmit buffer empty
		parameter.ulTBEIntCount++;
  	}

  	if (usEvent & (XSCC_INT_CTS+XSCC_INT_SYNC+XSCC_INT_DCD))
   {	// externnal interrupts
		if (usEvent & XSCC_INT_CTS)
		{
			parameter.ulCTSIntCount++;
		}
		if (usEvent & XSCC_INT_SYNC)
		{
			parameter.ulSYNCIntCount++;
		}
		if (usEvent & XSCC_INT_DCD)
		{
			parameter.ulDCDIntCount++;
		}
	}
	
	if (usEvent & (XSCC_INT_BRK+XSCC_INT_TMTUR_EOM+XSCC_INT_ZERO))
   {	// status conditions
		if (usEvent & XSCC_INT_BRK)
		{
			parameter.ulBRKIntCount++;
	  	}
		if (usEvent & XSCC_INT_TMTUR_EOM)
		{
			parameter.ulTMTURIntCount++;
	  	}
		if (usEvent & XSCC_INT_ZERO)
		{
			parameter.ulZEROIntCount++;
	  	}
	}
}


/****************************************************************************/
// main procedure of the task (called automatically by OsX after activating the task)
// poll receive state
void MAXEXPORT MainProc(void)
{
	UCHAR ucStatus;

	max_entry_proc();
	// poll receive available state	
	max_read_channel_uchar(parameter.hRR0, &ucStatus);
	if (ucStatus & XSCC_STATE_RCA)
	{	// read out the receive character
		max_read_channel_uchar(parameter.hRR0, &parameter.ucLastReceived);
	}
	max_exit_proc();
}

/****************************************************************************/
// Init task parameter
void MAXEXPORT AutoInit(void)
{
	max_entry_proc();
	
	max_init_lib(0);
	// get a handle for the X-MAX-1
	parameter.Error = max_connect_cpu (0, MAX_MYSELF, 0,&parameter.rcOsX, &parameter.hMyself);
	if (parameter.Error)
	   parameter.usFunc = 1;
	// default slot, layer and interface (modify it before calling start function)
	parameter.usLayerXSCC = 0;
	parameter.usSlotXSCC = 6;
	parameter.usChannel = 0;
	parameter.ulTBEIntCount = 0;
	parameter.ulRCAIntCount = 0;
	parameter.ulParityIntCount = 0;
	parameter.ulBRKIntCount = 0;
	parameter.ulTMTURIntCount = 0;
	parameter.ulCTSIntCount = 0;
	parameter.ulDCDIntCount = 0;
	parameter.ulSYNCIntCount = 0;
	parameter.ulZEROIntCount = 0;
	parameter.ulEOFCount = 0;
	parameter.ulFRAMEIntCount = 0;
	parameter.ulOVRIntCount = 0;
	max_exit_proc();
};


/****************************************************************************/
// init MDD and open channels
void MAXEXPORT Start(void)
{
	CPS_XSCC2	cps;
	ULONG			ulSize;

	max_entry_proc();
 
	parameter.Error = max_load_mdd (parameter.hMyself,
											  parameter.usSlotXSCC,
	                      			  parameter.usLayerXSCC, 0,
				          	 			  MDD_X_SCC_2, NULL,
											  &parameter.hMdd);
   if (parameter.Error)
   {
      parameter.usFunc = 2;
      max_exit_proc();
      return;
   }

	// open control channel to configure interface
	cps.usVersion 	  = 0;
	cps.usChannel    = parameter.usChannel;
	cps.usDevice     = DEVICE_CTRL;
   cps.usIndexFirst = XSCC_CTRL_PHYS_IFC;
   cps.usIndexLast  = cps.usIndexFirst;
	cps.usFlags      = _CP_EXCLUSIVE;
	cps.usType       = COM_RS232;
	cps.usReadMode   = IO_MODE_RAM;
	cps.usWriteMode  = IO_MODE_DIRECT;
	cps.usTRxC  	  = XSCC_SEL_NC;
	cps.usRTxC  	  = XSCC_SEL_PCLK;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hInterface);
   if (parameter.Error)
   {
      parameter.usFunc = 2;
      max_exit_proc();
      return;
   }

	// open channels to all ESCC write register
   cps.usDevice = DEVICE_REGISTER;
   cps.usFlags = _CP_EXCLUSIVE;
   cps.usMode = 0;
   cps.usReadMode = IO_MODE_RAM;
   cps.usWriteMode = IO_MODE_DIRECT;

   cps.usIndexFirst = 0;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR0);
   cps.usIndexFirst = 1;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR1);
   cps.usIndexFirst = 2;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR2);
   cps.usIndexFirst = 3;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR3);

   cps.usIndexFirst = 4;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR4);
   cps.usIndexFirst = 5;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR5);
   cps.usIndexFirst = 6;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR6);
   cps.usIndexFirst = 7;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR7);
   cps.usIndexFirst = 8;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR8);
   cps.usIndexFirst = 9;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR9);
   cps.usIndexFirst = 10;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR10);
   cps.usIndexFirst = 11;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR11);
   cps.usIndexFirst = 12;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR12);
   cps.usIndexFirst = 13;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR13);
   cps.usIndexFirst = 14;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR14);
   cps.usIndexFirst = 15;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR15);

   cps.usIndexFirst = XSCC_WR7_PRIME;
   cps.usIndexLast  = cps.usIndexFirst;
	parameter.Error  = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hWR7Prime);


	// open channels to all significant ESCC read register
   cps.usDevice = DEVICE_REGISTER;
   cps.usFlags = _CP_EXCLUSIVE;
   cps.usMode = 0;
   cps.usReadMode = IO_MODE_DIRECT;
   cps.usWriteMode = 0;

   cps.usIndexFirst = 0;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hRR0);
   cps.usIndexFirst = 1;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hRR1);
   cps.usIndexFirst = 2;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hRR2);

   cps.usIndexFirst = 6;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hRR6);

	cps.usIndexFirst = 7;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hRR7);

   cps.usIndexFirst = 10;
   cps.usIndexLast = cps.usIndexFirst;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, NULL, 0,
                                      &parameter.hRR10);

   if (parameter.Error)
   {
      parameter.usFunc = 2;
      max_exit_proc();
      return;
   }

	cps.usDevice     = DEVICE_HW_INT;
   cps.usIndexFirst = 0;
   cps.usIndexLast  = cps.usIndexFirst;
	cps.usFlags      = _CP_EXCLUSIVE + _CP_SYNC_CALLBACK;
	cps.usMode  	  = XSCC_INT_RCA + XSCC_INT_TBE;
	parameter.Error = max_open_channel(parameter.hMdd,
                                      sizeof(cps), &cps, ISR, 0,
                                      &parameter.hInterrupt);
   if (parameter.Error)
   {
      parameter.usFunc = 3;
      max_exit_proc();
      return;
   }

  	// programm ESCC to asynchronous mode 
	// 8 bit per character, 1 stop bit no parity, NRZ encoding
	// 9600 Baud generated by the baudrate generator
	max_write_channel_uchar(parameter.hWR3, XSCC_RX_8_BPC + XSCC_RX_ENABLE);
	max_write_channel_uchar(parameter.hWR4, XSCC_STOP_BIT_1 + XSCC_X16_CLOCK_MODE);
	max_write_channel_uchar(parameter.hWR5, XSCC_TX_8_BPC);
	max_write_channel_uchar(parameter.hWR10, XSCC_BIT_CODE_NRZ);
	max_write_channel_uchar(parameter.hWR11, XSCC_TMTCLK_BRG + XSCC_RCVCLK_BRG);
	// program baudrate generator time constant
	// TC = 14745600Hz/(2x16x9600) - 2 = 46 (s. SCC manual baud rate generator)
	max_write_channel_uchar(parameter.hWR12, 46);
	max_write_channel_uchar(parameter.hWR13, 0);
	// select and enable baud rate generator in 2 steps:
	parameter.Error = max_write_channel_uchar(parameter.hWR14, XSCC_BRG_SEL_PCLK);
	parameter.Error = max_write_channel_uchar(parameter.hWR14, XSCC_BRG_SEL_PCLK + XSCC_BRG_ENABLE);

	max_exit_proc();
};

/****************************************************************************/
// 
void MAXEXPORT Stop(void)
{
	max_entry_proc();
  
	max_exit_proc();
};


/****************************************************************************/
// transmit a number of bytes in polling mode
#pragma argsused
void MAXEXPORT Transmit (USHORT *pusInsize, UCHAR *pucData,
                         USHORT *pusOutsize, USHORT *pusDataOut)
{
   MAX_ERROR   Error;
	USHORT		i, usTimeout;
	UCHAR			ucStatus, ucData;

   max_entry_func();
   Error = ERR_OK;

  	// enable transmitter
	parameter.Error = max_write_channel_uchar(parameter.hWR5, XSCC_TX_8_BPC + XSCC_TX_ENABLE);
	for (i=0; i<*pusInsize; i++)
  	{	
		// in this sample we write one byte into the transmit buffer
		// then we wait for the transmit buffer to become empty 
		// before writing the next byte 
		// this can be optimized e.g. by using the FIFO 
		ucData = *pucData;
		pucData++;
		max_write_channel_uchar(parameter.hWR8, ucData);
		usTimeout = 100000;
		do 
	  	{	// poll the ALL SENT bit to be sure that the data was transmitted
			max_read_channel_uchar(parameter.hRR1, &ucStatus);
			usTimeout--;	
	  	}
		while	(((ucStatus & XSCC_ALL_SENT) == 0) && (usTimeout > 0));
		if (usTimeout == 0)
	  	{
			Error = ERR_INTERNAL_HARDWARE_TIMEOUT;
			break;
		}
	}
  	// disable transmitter
	max_write_channel_uchar(parameter.hWR5, XSCC_TX_8_BPC);

   max_exit_func(Error);
}


/****************************************************************************/
// initialize the Program Descriptor Table (PDT)
void main ()
{
	// the PDT-flags determine the task-type and
	// information how the data and parameter range are allocated
	
	//set flags for task type
	rcPDT.rcHead.usFlags         = MAX_NI_TASK +	MAXPDT_PARAMETER_IN_PROG;
	
	// interrupt source
	rcPDT.rcHead.usInterrupt     = 0;
	
	// the program number identifies the program uniquely
	rcPDT.rcHead.usProgNo        = PROGRAMM_NUMBER;
	
	// number of global task procedures in the PDT
	rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;
	
	// the following lines are just for documentation and information purpose
	rcPDT.rcHead.ucType          = 1;   // always !
	rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
	rcPDT.rcHead.cVersion        = VERSION;
	rcPDT.rcHead.cRevision       = REVISION;
	rcPDT.rcHead.ucCPU           = MAX_CPU486;
	rcPDT.rcHead.ucCoProc        = _NOCOPROC;
	rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
	rcPDT.rcHead.ucProgType      = 0;   // not used
	rcPDT.rcHead.ulRes1          = 0;
	rcPDT.rcHead.ulRes2          = 0;
	rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet
	
	// address and size of the tasks data range
	// only when MAXPDT_DATA_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulDataAdr       = 0;
	rcPDT.rcHead.ulDataSize      = 0;
	
	// address and size of the tasks parameter range
	// only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
	rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&parameter) + sizeof(MAX_TDT_TYPE);
	rcPDT.rcHead.usParSize = sizeof(parameter) - sizeof(MAX_TDT_TYPE);
	
	// initialize global procedure addresses in PDT
	rcPDT.main_proc        = MainProc;
	rcPDT.auto_init        = AutoInit;
	rcPDT.start        	  = Start;
	rcPDT.stop        	  = Stop;
	rcPDT.send        	  = Transmit;
	
	// transfer PDT information to the OsX
	max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
