/****************************************************************************
*	SSI.C	
*
*	PURPOSE
*		Sample Real Time program
* 		The program show handling of X-SSI-2 SSI channel. 
*	INSTALL-PARAMETER
*		Command for SORCUS INS-File: MAXINST file="SSI.EXE" no=9010 autoinit
*	SETTINGS
*		To use this program, you have to:
*			- install the MDD for X-SSI-2
*	COPYRIGHT
*		(C)2003 SORCUS Computer GmbH
*	HISTORY
*		- 2003-11-21 (JS) 1.A: First Version
*
****************************************************************************/

#pragma option -r-

// _RTLIB_ must be defined in every real time program
#define _RTLIB_

// including header files
#include "max_lib.h"
#include "x_error.h"
#include "x_mdd.h"
#include "dos.h"
#include "stdlib.h"

// program version
#define VERSION         	'1'
#define REVISION        	'A'
#define PROGRAM_NUMBER		0x9010

// status of program
#define PROG_INIT			0x1
#define PROG_START		0x2
#define PROG_RUN			0x3
#define PROG_STOP			0x4

#define SSI_READY       0x0
#define SSI_RUNNING 		0x1

// PDT (Program Descriptor Table) data structure
struct pdt_type
{
   MAX_PDT_HEAD    rcHead;
   void* MAXEXPORT main_proc;
   void* MAXEXPORT auto_init;
   void* MAXEXPORT start;
   void* MAXEXPORT stop;
} rcPDT;

// parameter range of the task
struct parameter_type
{
   MAX_TDT_TYPE rcTDT;
	MAX_ERROR    Error;				// error of program
	USHORT		 usStatus;			// status of program
	USHORT       usSlot;				// slot of module
	USHORT		 usLayer;			// layer of module
   MAXMODHND    hMyself;        	// handle for the CPU
	MAXMDDHND    hMdd;            // handle for the MDD          
	MAXCHLHND    hSsi;				// handle to SSI channel
	MAX_OS_INFO  rcOsX;				// OsX
	ULONG        ulSsiData;			// SSI data	
} rcParameter;

// Prototypes
void MAXEXPORT MainProc (void);
void MAXEXPORT AutoInit (void);
void MAXEXPORT Start (void);
void MAXEXPORT Stop (void);



/*************************************************************************
*			AutoInit
*************************************************************************/
void MAXEXPORT AutoInit(void)
{
	max_entry_proc();

   // init library
	rcParameter.Error = max_init_lib(0);
   if (rcParameter.Error == ERR_OK) 
	{
      // get a handle for the X-MAX-1
      rcParameter.Error = max_connect_cpu (0, MAX_MYSELF, 0, &rcParameter.rcOsX, &rcParameter.hMyself);
   }

	// set slot and layer of X-SSI-2
	rcParameter.usSlot = 6;
	rcParameter.usLayer = 0;

	// status
	rcParameter.usStatus = PROG_INIT;

   max_exit_proc();
};



/*************************************************************************
*			Start
*************************************************************************/
void MAXEXPORT Start(void)
{
   USHORT         i;
   CPS_XSSI2_A   rcCps;

   max_entry_proc();


	// check error
	if (rcParameter.Error == ERR_OK)
	{
		// get a handle for the MDD
		rcParameter.Error = max_load_mdd (rcParameter.hMyself, rcParameter.usSlot, rcParameter.usLayer, 0, MDD_X_SSI_2, NULL, &rcParameter.hMdd);

	   if (rcParameter.Error == ERR_OK) 
		{
			// fill in CPS for SSI channel
			rcCps.usVersion = 1;
			rcCps.usDevice = DEVICE_SSI;
			rcCps.usIndexFirst = 0;
			rcCps.usIndexLast = 0;
			rcCps.usFlags = _CP_EXCLUSIVE;
			rcCps.usReadMode = IO_MODE_DIRECT;
			rcCps.usWriteMode = 0;
			rcCps.usMode = XSSI2_AUTO_MODE;
			rcCps.usDataMode = 0;
			rcCps.ulClock = 100000;
			rcCps.usBit = 30;

			// open channel
		   rcParameter.Error = max_open_channel (rcParameter.hMdd, sizeof (rcCps), &rcCps, NULL, 0, &rcParameter.hSsi);

	     	if (rcParameter.Error == ERR_OK)
		  	{
				// activate task
				rcParameter.Error = max_wakeup_ni_task(rcParameter.hMyself, rcParameter.rcTDT.usTaskNo, 1);
		  	}
		}
	}

	// status
	rcParameter.usStatus = PROG_START;

   max_exit_proc();
}



/*************************************************************************
*			MainProc
*************************************************************************/
void MAXEXPORT MainProc(void)
{
   UCHAR ucLed, ucOutData;
	MAX_ERROR Error;

   max_entry_proc();

	// status
	rcParameter.usStatus = PROG_RUN;

	// read data
	rcParameter.Error = max_read_channel_ulong(rcParameter.hSsi, &rcParameter.ulSsiData);

	if (rcParameter.Error != ERR_OK)
	{
		max_clear_error();
	}

 	max_exit_proc();
}



/*************************************************************************
*			Stop
*************************************************************************/
void MAXEXPORT Stop(void)
{
   max_entry_proc();

	// deactivate task
   rcParameter.Error = max_sleep_task (rcParameter.hMyself, rcParameter.rcTDT.usTaskNo);

	// close channels
	rcParameter.Error = max_close_channel(&rcParameter.hSsi);

	// status
	rcParameter.usStatus = PROG_STOP;

   max_exit_proc();
}




/*************************************************************************
*
*************************************************************************/
void main ()
{
   // the PDT-flags determine the task-type and
   // information how the data and parameter range are allocated
   rcPDT.rcHead.usFlags         = MAX_NI_TASK + MAXPDT_PARAMETER_IN_PROG; // parameter are part of the program

   // interrupt source (0 for NI-tasks)
   rcPDT.rcHead.usInterrupt     = 0;

   // the program number identifies the program uniquely
   rcPDT.rcHead.usProgNo        = PROGRAM_NUMBER;

   // number of global task procedures in the PDT
   rcPDT.rcHead.usProcedures    = (sizeof(rcPDT) - sizeof (MAX_PDT_HEAD)) / 4;

   // the following lines are just for documentation and information purpose
   rcPDT.rcHead.ucType          = 1;   // always !
   rcPDT.rcHead.ucSize          = sizeof(MAX_PDT_HEAD);
   rcPDT.rcHead.cVersion        = VERSION;
   rcPDT.rcHead.cRevision       = REVISION;
   rcPDT.rcHead.ucCPU           = MAX_CPU486;
   rcPDT.rcHead.ucCoProc        = _NOCOPROC;
   rcPDT.rcHead.ucLanguage      = MAXRT_LANGUAGE_C;
   rcPDT.rcHead.ucProgType      = 0;   // not used
   rcPDT.rcHead.ulRes1          = 0;
   rcPDT.rcHead.ulRes2          = 0;
   rcPDT.rcHead.ulHyperAdr      = 0;   // feature not available yet

   // address and size of the tasks data range
   // only when MAXPDT_DATA_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulDataAdr       = 0;
   rcPDT.rcHead.ulDataSize      = 0;

   // address and size of the tasks parameter range
   // only when MAXPDT_PARAMETER_IN_PROG is set in PDT-flags
   rcPDT.rcHead.ulParAdr  = max_get_physical_addr(&rcParameter) + sizeof(MAX_TDT_TYPE);
   rcPDT.rcHead.usParSize = sizeof(rcParameter) - sizeof(MAX_TDT_TYPE);

   // initialize global procedure addresses in PDT
   rcPDT.main_proc        = MainProc;
   rcPDT.auto_init        = AutoInit;
   rcPDT.start            = Start;
   rcPDT.stop             = Stop;
   // transfer PDT information to the OsX
   max_init_program ((MAX_PDT_HEAD*)&rcPDT);
}
