/****************************************************************************
*	SSICONT.CPP	
*
*	PURPOSE
*		Sample program
* 		The program show handling of X-SSI-2/M module-device-driver.
*       Be sure that you will use the multithreaded version of the runtime
*       library.
*	COPYRIGHT
*		(C) 2003 SORCUS Computer GmbH
*	HISTORY
*		- 2003-10-21 (JS) First Version
****************************************************************************/

// callback.cpp : Definiert den Einsprungpunkt fr die Konsolenanwendung.
//

#include "windows.h"
#include "iostream.h"
#include "conio.h"

// include SORCUS header files
#include "max_lib.h"
#include "x_mdd.h"

// add SORCUS maxw32.lib
#pragma comment( lib, "c:\\sorcus\\maxlib\\pc\\win32\\msvc\\maxw32" )


#define STATUS_READY   1
#define STATUS_RUNNING 2

// some global variables
USHORT g_usBoardNo, g_usCpuSlot, g_usCpuLayer, g_usXssi2Slot, g_usXssi2Layer;
ULONG g_ulCounter;
MAXMODHND g_hCpu;
MAXMDDHND g_hXssi2Mdd;
MAXCHLHND g_hSsi;




//*****************************************************************************
// HandleError
/**
* 
* @param    pcText
* @param    Error
* @return   -
*/
//*****************************************************************************
void HandleError(CHAR* pcText, MAX_ERROR Error)
{
    CHAR acMessage[200];
    USHORT usSize = 200;

    max_clear_error();
    max_get_error_message(Error, &usSize, acMessage);
    cout << "-------------------------------------------------------------------" << endl;
    cout << "Error at        : " << pcText << endl;
    cout << "Error-Code      : " << hex << Error << dec << "hex" << endl;
    cout << "Error-Message   : " << acMessage << endl;
    cout << "-------------------------------------------------------------------" << endl;
    return;
}



//*****************************************************************************
// MyCallbackFunction
/**
*
* @param	handle
* @param	param
* @param	size
* @param	pData
* @return	MAX_CALLBACK
*/
//*****************************************************************************
MAX_CALLBACK MyCallbackFunction(MAXCHLHND handle, ULONG param, ULONG size, void* pData)
{
    ULONG i, *pulData;
    MAX_ERROR Error;

    g_ulCounter++;
    cout << "Callback-Funktion has been called " << g_ulCounter << " times" << endl;

    // get data
    pulData = (ULONG*)pData;
    if (size != (2 * sizeof(ULONG)))
    {
        cout << "Error wrong data size!" << endl;
    }
    else
    {
        cout << "Status: " << hex << *pulData++ << "h " << dec;
        cout << "Data: " << hex << *pulData << "h " << dec;
        cout << endl;
    }
}



//*****************************************************************************
// main
/**
*
* @param    argc
* @param    argv
* @return   int
*/
//*****************************************************************************
int main(int argc, char* argv[])
{
    USHORT i;
    USHORT usIndex=0;
    MAX_ERROR Error;
    MAX_OS_INFO rcOs;
    CPS_XSSI2_A rcCps;

    // init global variables
    g_usBoardNo = 0;
    g_usCpuSlot = 1;
    g_usCpuLayer = 0;
    g_usXssi2Slot = 5;
    g_usXssi2Layer = 0;
    g_ulCounter = 0;

    // init library
    Error = max_init_lib(0);
    
    // reset board
    Error = max_reset_board(g_usBoardNo, 20);
    if (Error != ERR_OK)
    {
        HandleError("max_reset_board", Error);
        _getch();
        return 1;
    }
    
    // connect cpu
    Error = max_connect_cpu (g_usBoardNo, g_usCpuSlot, g_usCpuLayer, &rcOs, &g_hCpu);
    if (Error != ERR_OK)
    {
        HandleError("max_connect_cpu", Error);
        _getch();
        return 1;
    }

    // load osx
    if (g_usCpuSlot != 0)
    {
        Error = max_load_osx (g_hCpu, "ROM");
        if (Error != ERR_OK)
        {
            HandleError("max_load_osx", Error);
            _getch();
            return 1;
        }
    }

    // load mdd
    Error = max_load_mdd (g_hCpu, g_usXssi2Slot, g_usXssi2Layer, 0, MDD_X_SSI_2, NULL, &g_hXssi2Mdd);
    if (Error != ERR_OK)
    {
        HandleError("max_load_mdd", Error);
        _getch();
        return 1;
    }

    // open ssi channel
    rcCps.usVersion = 1;
	rcCps.usDevice = DEVICE_SSI;
	rcCps.usIndexFirst = 0;
	rcCps.usIndexLast = 0;
	rcCps.usFlags = _CP_EXCLUSIVE;
	rcCps.usReadMode = IO_MODE_DIRECT;
	rcCps.usWriteMode = 0;
	rcCps.usMode = XSSI2_CONTINUOUS_MODE;
	rcCps.usDataMode = 0;
	rcCps.ulClock = 100000;
	rcCps.usBit = 30;

    Error = max_open_channel( g_hXssi2Mdd, sizeof (rcCps), &rcCps, MyCallbackFunction, 0, &g_hSsi );

    if (Error != ERR_OK)
    {
        HandleError("max_open_channel", Error);
        _getch();
        return 1;
    }

    for (i=0; i<3000; i++)
    {
        Sleep(1000);
    }

    // close channel
    Error = max_close_channel(&g_hSsi);
    if (Error != ERR_OK)
    {
        HandleError("max_close_channel", Error);
        _getch();
        return 1;
    }

    cout << "End of example" << endl;
    _getch();

    return 0;
}
